@@ -4,40 +4,50 @@
 /* on Mingw-w64 defines the MING64_ version */
 #include <_mingw.h>
 
-/* Define to one of `_getb67', `GETB67', `getb67' for Cray-2 and Cray-YMP
-   systems. This function is required for `alloca.c' support on those systems.
-   */
-/* #undef CRAY_STACKSEG_END */
+/* Version of C Compiler */
+#define CC_VER "@CC_VER@"
 
-/* Define to 1 if using `alloca.c'. */
+/* Define to 1 if using 'alloca.c'. */
 /* #undef C_ALLOCA */
 
+/* C stack direction: 1 (down) or -1 (up) */
+#define C_STACK_DIRECTION 1
+
 /* Define to 1 if translation of program messages to the user's native
    language is requested. */
 #define ENABLE_NLS 1
 
 /* Define to dummy `main' function (if any) required to link to the Fortran
    libraries. */
-/* #undef F77_DUMMY_MAIN */
-
-/* Define to a macro mangling the given C identifier (in lower and upper
-   case), which must not contain underscores, for linking with Fortran. */
-#define F77_FUNC(name,NAME) name ## _
-
-/* As F77_FUNC, but for C identifiers containing underscores. */
-#define F77_FUNC_(name,NAME) name ## _
+/* #undef FC_DUMMY_MAIN */
 
 /* Define if F77 and FC dummy `main' functions are identical. */
 /* #undef FC_DUMMY_MAIN_EQ_F77 */
 
+/* Define to a macro mangling the given C identifier (in lower and upper
+   case), which must not contain underscores, for linking with Fortran. */
+#define FC_FUNC(name,NAME) name ## _
+
+/* As FC_FUNC, but for C identifiers containing underscores. */
+#define FC_FUNC_(name,NAME) name ## _
+
+/* C type used for Fortran character lengths */
+/* will be size_t for gfortran >= 8, int before */
+/* #undef FC_LEN_T */
+#if defined __GNUC__ && __GNUC__ >= 8
+#define FC_LEN_T size_t
+#endif
+
+/* Version of Fortran Compiler */
+#define FC_VER "@FC_VER@"
+
 /* Define to 1 if you have the `access' function. */
 #define HAVE_ACCESS 1
 
-/* Define to 1 if you have `alloca', as a function or macro. */
+/* Define to 1 if you have 'alloca', as a function or macro. */
 #define HAVE_ALLOCA 1
 
-/* Define to 1 if you have <alloca.h> and it should be used (not on Ultrix).
-   */
+/* Define to 1 if <alloca.h> works. */
 /* #undef HAVE_ALLOCA_H */
 
 /* Define if you have the Aqua headers and libraries, and want to include
@@ -60,6 +70,7 @@
 /* #undef HAVE_ARPA_INET_H */
 
 /* Define to 1 if you have the `asprintf' function. */
+/* available on Mingw-w64 with _GNU_SOURCE */
 /* #undef HAVE_ASPRINTF */
 
 /* Define to 1 if you have the `atan2pi' function. */
@@ -68,14 +79,11 @@
 /* Define to 1 if you have the `atanpi' function. */
 /* #undef HAVE_ATANPI */
 
-/* Define if you have BSD networking headers and libraries. */
-/* #undef HAVE_BSD_NETWORKING */
-
 /* Define to 1 if the compiler understands __builtin_expect. (For intl) */
 #define HAVE_BUILTIN_EXPECT 1
 
 /* Define to 1 if you have the <bzlib.h> header file. */
-/* #undef HAVE_BZLIB_H */
+#define HAVE_BZLIB_H 1
 
 /* Define to 1 if you have the `cabs' function. */
 #define HAVE_CABS 1
@@ -114,8 +122,8 @@
    CoreFoundation framework. (For intl) */
 /* #undef HAVE_CFLOCALECOPYCURRENT */
 
-/* Define to 1 if you have the OS X function CFPreferencesCopyAppValue in
-   the CoreFoundation framework. (For intl) */
+/* Define to 1 if you have the OS X function CFPreferencesCopyAppValue in the
+   CoreFoundation framework. (For intl) */
 /* #undef HAVE_CFPREFERENCESCOPYAPPVALUE */
 
 /* Define to 1 if you have the `chdir' function. */
@@ -154,7 +162,14 @@
 #define HAVE_CTAN 1
 
 /* Define to 1 if you have the `ctanh' function. */
-#define HAVE_CTANH 1
+/* The ctanh function does not work properly, at R level "tanh(356+0i)"
+   returns "1+NaNi", but it should return "1+0i". */
+/* #undef HAVE_CTANH */
+
+/* Define to 1 if you have the <curl/curl.h> header file.
+   Set on the command line where supported (used via USE_LIBCURL)
+*/
+/* #undef HAVE_CURL_CURL_H */
 
 /* Define if the GNU dcgettext() function is already present or preinstalled.
    */
@@ -164,6 +179,14 @@
    */
 #define HAVE_DECL_ALLOCA 0
 
+/* Define to 1 if you have the declaration of `dladdr', and to 0 if you don't.
+   */
+#define HAVE_DECL_DLADDR 0
+
+/* Define to 1 if you have the declaration of `dlsym', and to 0 if you don't.
+   */
+#define HAVE_DECL_DLSYM 0
+
 /* Define to 1 if you have the declaration of `feof_unlocked', and to 0 if you
    don't. (For intl) */
 #define HAVE_DECL_FEOF_UNLOCKED 0
@@ -196,12 +219,22 @@
    don't. */
 #define HAVE_DECL_REALPATH 0
 
+/* Define to 1 if you have the declaration of `RTLD_DEFAULT', and to 0 if you
+   don't. */
+#define HAVE_DECL_RTLD_DEFAULT 0
+
+/* Define to 1 if you have the declaration of `RTLD_NEXT', and to 0 if you
+   don't. */
+#define HAVE_DECL_RTLD_NEXT 0
+
 /* Define to 1 if you have the declaration of `siglongjmp', and to 0 if you
    don't. */
+/* from psignal.h */
 #define HAVE_DECL_SIGLONGJMP 1
 
 /* Define to 1 if you have the declaration of `sigsetjmp', and to 0 if you
    don't. */
+/* from psignal.h */
 #define HAVE_DECL_SIGSETJMP 1
 
 /* Define to 1 if you have the declaration of `SIZE_MAX', and to 0 if you
@@ -218,6 +251,7 @@
 
 /* Define to 1 if you have the declaration of `vasprintf', and to 0 if you
    don't. */
+/* available on Mingw-w64 with _GNU_SOURCE */
 #define HAVE_DECL_VASPRINTF 0
 
 /* Define to 1 if you have the declaration of `_snprintf', and to 0 if you
@@ -232,11 +266,14 @@
    */
 #define HAVE_DIRENT_H 1
 
+/* Define to 1 if you have the `dladdr' function. */
+/* #undef HAVE_DLADDR */
+
 /* Define to 1 if you have the <dlfcn.h> header file. */
 /* #undef HAVE_DLFCN_H */
 
-/* Define to 1 if you have the <dl.h> header file. */
-/* #undef HAVE_DL_H */
+/* Define to 1 if you have the `dlsym' function. */
+/* #undef HAVE_DLSYM */
 
 /* Define to 1 if you have the <elf.h> header file. */
 /* #undef HAVE_ELF_H */
@@ -269,6 +306,9 @@
 /* Define to 1 if you have the `fdopen' function. */
 #define HAVE_FDOPEN 1
 
+/* Define to 1 if you have the <features.h> header file. */
+/* #undef HAVE_FEATURES_H */
+
 /* Define to 1 if you have the <floatingpoint.h> header file. */
 /* #undef HAVE_FLOATINGPOINT_H */
 
@@ -276,13 +316,12 @@
    can do arithmetic on the latter. */
 #define HAVE_FORTRAN_DOUBLE_COMPLEX 1
 
-/* Define to 1 if you have the <fpu_control.h> header file. */
-/* #undef HAVE_FPU_CONTROL_H */
-
 /* Define to 1 if fseeko (and presumably ftello) exists and is declared. */
+/* available on Mingw-w64 */
 /* #undef HAVE_FSEEKO */
 
 /* Define to 1 if you have the `ftello' function. */
+/* available on Mingw-w64 */
 /* #undef HAVE_FTELLO */
 
 /* Define to 1 if you have the `ftruncate' function. */
@@ -306,12 +345,18 @@
 /* Define to 1 if you have the `getgrgid' function. */
 /* #undef HAVE_GETGRGID */
 
+/* Define to 1 if you have the `getline' function. */
+/* #undef HAVE_GETLINE */
+
 /* Define to 1 if you have the `getpagesize' function. */
 #define HAVE_GETPAGESIZE 1
 
 /* Define to 1 if you have the `getpriority' function. */
 /* #undef HAVE_GETPRIORITY */
 
+/* Define to 1 if you have the `getpwnam' function. */
+/* #undef HAVE_GETPWNAM */
+
 /* Define to 1 if you have the `getpwuid' function. */
 /* #undef HAVE_GETPWUID */
 
@@ -358,15 +403,15 @@
 /* Define if you have the `iconvlist' function. */
 #define HAVE_ICONVLIST 1
 
+/* Define if `iconv' accepts "CP1252". */
+#define HAVE_ICONV_CP1252 1
+
 /* Define to 1 if you have the <iconv.h> header file. */
 #define HAVE_ICONV_H 1
 
 /* Define to 1 if the system has the type `int64_t'. */
 #define HAVE_INT64_T 1
 
-/* Define if you have support for ftp/http access. */
-#define HAVE_INTERNET 1
-
 /* Define if you have the 'intmax_t' type in <stdint.h> or <inttypes.h>. (For
    intl) */
 #define HAVE_INTMAX_T 1
@@ -390,9 +435,6 @@
 /* Define to 1 if you have the `isnan' function. */
 #define HAVE_ISNAN 1
 
-/* Define to 1 if you have the `iswblank' function. */
-#define HAVE_ISWBLANK 1
-
 /* Define to 1 if you have the `iswctype' function. */
 #define HAVE_ISWCTYPE 1
 
@@ -417,12 +459,22 @@
 /* Define if your <locale.h> file defines LC_MESSAGES. */
 /* #undef HAVE_LC_MESSAGES */
 
-/* Define to 1 if you have the `cc_dynamic' library (-lcc_dynamic). */
-/* #undef HAVE_LIBCC_DYNAMIC */
+/* Define if your system has libcurl >= 7.28.0 with support for https.
+   Set on the command line where supported.  (used via USE_LIBCURL)
+ */
+/* #undef HAVE_LIBCURL */
 
 /* Define if __libc_stack_end is visible. */
 /* #undef HAVE_LIBC_STACK_END */
 
+/* Define to 1 if you have libdeflate headers and library. */
+/* #undef HAVE_LIBDEFLATE */
+#define HAVE_LIBDEFLATE 1
+
+/* Define to 1 if you have the <libdeflate.h> header file. */
+/* #undef HAVE_LIBDEFLATE_H */
+#define HAVE_LIBDEFLATE_H 1
+
 /* Define to 1 if you have the `dl' library (-ldl). */
 /* #undef HAVE_LIBDL */
 
@@ -439,7 +491,7 @@
 /* #undef HAVE_LIBREADLINE */
 
 /* Define to 1 if you have the `rt' library (-lrt). */
-#undef HAVE_LIBRT
+/* #undef HAVE_LIBRT */
 
 /* Define to 1 if you have the `sunmath' library (-lsunmath). */
 /* #undef HAVE_LIBSUNMATH */
@@ -450,6 +502,9 @@
 /* Define to 1 if you have the `termlib' library (-ltermlib). */
 /* #undef HAVE_LIBTERMLIB */
 
+/* Define to 1 if you have the `tinfo' library (-ltinfo). */
+/* #undef HAVE_LIBTINFO */
+
 /* Define to 1 if you have the `tk' library (-ltk). */
 /* #undef HAVE_LIBTK */
 
@@ -463,7 +518,8 @@
 #define HAVE_LOCALE_H 1
 
 /* Define to 1 if you have the `localtime_r' function. */
-#define HAVE_LOCALTIME_R 1
+/* available on Mingw-w64 with _POSIX_THREAD_SAFE_FUNCTIONS */
+/* #undef HAVE_LOCALTIME_R */
 
 /* Define to 1 if you have the `log10' function. */
 #define HAVE_LOG10 1
@@ -484,13 +540,10 @@
 #define HAVE_LONG_LONG_INT 1
 
 /* Define if your system has lzma >= 5.0.3. */
-/* #undef HAVE_LZMA */
+#define HAVE_LZMA 1
 
 /* Define to 1 if you have the <lzma.h> header file. */
-/* #undef HAVE_LZMA_H */
-
-/* Define to 1 if you have the `matherr' function. */
-/* #undef HAVE_MATHERR */
+#define HAVE_LZMA_H 1
 
 /* Define to 1 if you have the `mbrtowc' function. */
 #define HAVE_MBRTOWC 1
@@ -501,14 +554,14 @@
 /* Define to 1 if you have the `mbstowcs' function. */
 #define HAVE_MBSTOWCS 1
 
-/* Define to 1 if you have the <memory.h> header file. */
-#define HAVE_MEMORY_H 1
-
 /* Define to 1 if you have the `mempcpy' function. */
 #if defined(__MINGW64_VERSION_MAJOR) && __MINGW64_VERSION_MAJOR >= 2
 # define HAVE_MEMPCPY 1
 #endif
 
+/* Define to 1 if you have the <minix/config.h> header file. */
+/* #undef HAVE_MINIX_CONFIG_H */
+
 /* Define to 1 if you have the `mkdtemp' function. */
 /* #undef HAVE_MKDTEMP */
 
@@ -556,9 +609,15 @@
 // #define HAVE_OPENMP 1
 #endif
 
+/* Define if your OpenMP 4 implementation fully supports SIMD reduction */
+#define HAVE_OPENMP_SIMDRED 1
+
 /* Define to 1 if you have pangocairo. */
 /* #undef HAVE_PANGOCAIRO */
 
+/* Define if your system has pcre2. */
+#define HAVE_PCRE2 1
+
 /* Define to 1 if you have the <pcre.h> header file. */
 /* #undef HAVE_PCRE_H */
 
@@ -577,9 +636,12 @@
 
 /* Define if your printf() function supports format strings with positions.
    (For intl) */
+/* UCRT printf does not support positions,
+   but the TRIO replacement does (trioremap.h) */
 #define HAVE_POSIX_PRINTF 1
 
 /* Define if you have POSIX.1 compatible sigsetjmp/siglongjmp. */
+/* from psignal.h */
 #define HAVE_POSIX_SETJMP 1
 
 /* Define to 1 if you have the `powl' function. */
@@ -588,6 +650,10 @@
 /* Define to 1 if you have the `pown' function. */
 /* #undef HAVE_POWN */
 
+/* Define if have support for POSIX threads. */
+/* may be available on Mingw-w64 with winpthreads */
+/* #undef HAVE_PTHREAD */
+
 /* Define if the <pthread.h> defines PTHREAD_MUTEX_RECURSIVE. (For intl) */
 /* #undef HAVE_PTHREAD_MUTEX_RECURSIVE */
 
@@ -619,23 +685,30 @@
 /* Define to 1 if you have the `realpath' function. */
 /* #undef HAVE_REALPATH */
 
-/* Define to 1 if you have the `rint' function. */
-#define HAVE_RINT 1
-
 /* Define to 1 if you have the `rintl' function. */
 #define HAVE_RINTL 1
 
 /* Define to 1 if you have the `rl_completion_matches' function. */
 /* #undef HAVE_RL_COMPLETION_MATCHES */
 
+/* Define to 1 if you have the `rl_resize_terminal' function. */
+/* #undef HAVE_RL_RESIZE_TERMINAL */
+
+/* Define to 1 if you have the `rl_sort_completion_matches' function. */
+/* #undef HAVE_RL_SORT_COMPLETION_MATCHES */
+
 /* Define to 1 if you have the `sched_getaffinity' function. */
-#undef HAVE_SCHED_GETAFFINITY
+/* #undef HAVE_SCHED_GETAFFINITY */
 
 /* Define to 1 if you have the <sched.h> header file. */
-#undef HAVE_SCHED_H
+/* may be available on Mingw-w64 with winpthreads */
+/* #undef HAVE_SCHED_H */
 
 /* Define to 1 if you have the `sched_setaffinity' function. */
-#undef HAVE_SCHED_SETAFFINITY
+/* #undef HAVE_SCHED_SETAFFINITY */
+
+/* Define to 1 if you have the `select' function. */
+/* #undef HAVE_SELECT */
 
 /* Define to 1 if you have the `setenv' function. */
 /* #undef HAVE_SETENV */
@@ -646,6 +719,9 @@
 /* Define to 1 if you have the `setlocale' function. */
 #define HAVE_SETLOCALE 1
 
+/* Define to 1 if you have the `setrlimit' function. */
+/* #undef HAVE_SETRLIMIT */
+
 /* Define to 1 if you have the `sigaction' function. */
 /* #undef HAVE_SIGACTION */
 
@@ -661,15 +737,15 @@
 /* Define to 1 if you have the `snprintf' function. */
 #define HAVE_SNPRINTF 1
 
-/* Define if you have support for sockets. */
-#define HAVE_SOCKETS 1
-
 /* Define to 1 if the system has the type `stack_t'. */
 /* #undef HAVE_STACK_T */
 
 /* Define to 1 if you have the `stat' function. */
 #define HAVE_STAT 1
 
+/* Define to 1 if you have the <stdalign.h> header file. */
+#define HAVE_STDALIGN_H 1
+
 /* Define to 1 if you have the <stdarg.h> header file. */
 #define HAVE_STDARG_H 1
 
@@ -686,6 +762,9 @@
    uintmax_t. (For intl) */
 #define HAVE_STDINT_H_WITH_UINTMAX 1
 
+/* Define to 1 if you have the <stdio.h> header file. */
+#define HAVE_STDIO_H 1
+
 /* Define to 1 if you have the <stdlib.h> header file. */
 #define HAVE_STDLIB_H 1
 
@@ -774,19 +853,30 @@
 
 /* Define if you have the Tcl/Tk headers and libraries and want Tcl/Tk support
    to be built. */
+/* Tcl/Tk bundle is by default expected in Tcl directory next to src */
 #define HAVE_TCLTK 1
 
+/* Define to 1 if you have the <thread.h> header file. */
+/* #undef HAVE_THREAD_H */
+
+/* Define to 1 if you have the `thr_stksegment' function. */
+/* #undef HAVE_THR_STKSEGMENT */
+
 /* Define this if libtiff is available. */
 #define HAVE_TIFF 1
 
 /* Define to 1 if you have the <tiffio.h> header file. */
-/* #undef HAVE_TIFFIO_H */
+#define HAVE_TIFFIO_H 1
+
+/* Define to 1 if you have the `tilde_expand_word' function. */
+/* #undef HAVE_TILDE_EXPAND_WORD */
 
 /* Define to 1 if you have the `times' function. */
-#define HAVE_TIMES 1
+/* #undef HAVE_TIMES */
 
 /* Define to 1 if you have the `timespec_get' function. */
-/* #undef HAVE_TIMESPEC_GET */
+/* available since Rtools43, but not in Rtools42 */
+/* #define HAVE_TIMESPEC_GET 1 */
 
 /* Define to 1 if your 'struct tm' has tm_gmtoff. */
 /* #undef HAVE_TM_GMTOFF */
@@ -794,6 +884,12 @@
 /* Define to 1 if your 'struct tm' has tm_zone. */
 /* #undef HAVE_TM_ZONE */
 
+/* Define if your system has tre. */
+/* #undef HAVE_TRE */
+
+/* Define to 1 if you have the <tre/tre.h> header file. */
+/* #undef HAVE_TRE_TRE_H */
+
 /* Define to 1 if you have the `tsearch' function. */
 #if defined(__MINGW64_VERSION_MAJOR) && __MINGW64_VERSION_MAJOR >= 2
 #define HAVE_TSEARCH 1
@@ -823,23 +919,28 @@
 #define HAVE_UNSIGNED_LONG_LONG_INT 1
 
 /* Define to 1 if you have the `utime' function. */
+/* may be available on Mingw-w64 */
 /* #undef HAVE_UTIME */
 
+/* Define to 1 if you have the `utimensat' function. */
+/* #undef HAVE_UTIMENSAT */
+
 /* Define to 1 if you have the `utimes' function. */
 /* #undef HAVE_UTIMES */
 
 /* Define to 1 if you have the <utime.h> header file. */
+/* may be available on Mingw-w64 */
 /* #undef HAVE_UTIME_H */
 
+/* Define to 1 if you have the <valgrind/memcheck.h> header file. */
+/* #undef HAVE_VALGRIND_MEMCHECK_H */
+
 /* Define to 1 if you have the `vasprintf' function. */
 #define HAVE_VASPRINTF 1
 
 /* Define to 1 if you have the `va_copy' function. */
 #define HAVE_VA_COPY 1
 
-/* Defined if framework vecLib is present */
-/* #undef HAVE_VECLIB_FW */
-
 /* Define to 1 or 0, depending whether the compiler supports simple visibility
    declarations. (For intl) */
 #define HAVE_VISIBILITY 1
@@ -871,6 +972,9 @@
 /* Define to 1 if you have the `wcstombs' function. */
 #define HAVE_WCSTOMBS 1
 
+/* Define to 1 if you have the `wcswidth' function. */
+/* #undef HAVE_WCSWIDTH */
+
 /* Define to 1 if you have the `wctrans' function. */
 #define HAVE_WCTRANS 1
 
@@ -883,18 +987,22 @@
 /* Define to 1 if you have the <wctype.h> header file. */
 #define HAVE_WCTYPE_H 1
 
+/* Define to 1 if you have the `wcwidth' function. */
+/* #undef HAVE_WCWIDTH */
+
 /* Define if you have the 'wint_t' type. (For intl) */
 #define HAVE_WINT_T 1
 
-/* Define if your mktime works correctly outside 1902-2037. */
-#define HAVE_WORKING_64BIT_MKTIME 1
-
 /* Define to 1 if you have cairo. */
 /* #undef HAVE_WORKING_CAIRO */
 
 /* Define if calloc(0) returns a null pointer. */
 #define HAVE_WORKING_CALLOC 1
 
+/* Define if ctanh() exists and is working correctly. */
+/* ctanh is not working correctly on Windows, see HAVE_CTANH below. */
+/* #undef HAVE_WORKING_CTANH */
+
 /* Define if your ftell works correctly on files opened for append. */
 /* #undef HAVE_WORKING_FTELL */
 
@@ -904,9 +1012,24 @@
 /* Define if log1p() exists and is accurate enough. */
 #define HAVE_WORKING_LOG1P 1
 
+/* Define if your mktime works correctly after 2037. */
+/* using internal-tzcode */
+#define HAVE_WORKING_MKTIME_AFTER_2037 1
+
+/* Define if your mktime works correctly before 1902. */
+/* using internal-tzcode */
+#define HAVE_WORKING_MKTIME_BEFORE_1902 1
+
+/* Define if your mktime works correctly before 1970. */
+/* using internal-tzcode */
+#define HAVE_WORKING_MKTIME_BEFORE_1970 1
+
 /* Define if sigaction() is complete enough for R's usage */
 /* #undef HAVE_WORKING_SIGACTION */
 
+/* Define to 1 if you have cairo including Xlib. */
+/* #undef HAVE_WORKING_X11_CAIRO */
+
 /* Define if you have the X11 headers and libraries, and want the X11 GUI to
    be built. */
 /* #undef HAVE_X11 */
@@ -914,39 +1037,42 @@
 /* Define if you have the X11/Xmu headers and libraries. */
 /* #undef HAVE_X11_Xmu */
 
+/* Define to 1 if you have the `__cospi' function. */
+/* #undef HAVE___COSPI */
+
 /* Define to 1 if you have the `__fsetlocking' function. */
 /* #undef HAVE___FSETLOCKING */
 
+/* Define to 1 if you have the `__sinpi' function. */
+/* #undef HAVE___SINPI */
+
+/* Define to 1 if you have the `__tanpi' function. */
+/* #undef HAVE___TANPI */
+
 /* Define as const if the declaration of iconv() needs const. */
+/* to match fixed/h/iconv.h */
 #define ICONV_CONST const
 
 /* Define if you have IEEE 754 floating point arithmetic. */
 #define IEEE_754 1
 
 /* Define if integer division by zero raises signal SIGFPE. (For intl) */
-#define INTDIV0_RAISES_SIGFPE 1
+#define INTDIV0_RAISES_SIGFPE 0
 
 /* Define if you have 32 bit ints. */
 #define INT_32_BITS 1
 
-/* Define to the sub-directory in which libtool stores uninstalled libraries.
-   */
+/* Define to the sub-directory where libtool stores uninstalled libraries. */
 #define LT_OBJDIR ".libs/"
 
 /* Define if mktime sets errno. */
 #define MKTIME_SETS_ERRNO 1
 
-/* Define if your system needs __setfpucw() to control FPU rounding. This was
-   used to control floating point precision, rounding and floating point
-   exceptions on older Linux systems. As of GLIBC 2.1 this function is not
-   used anymore. */
-/* #undef NEED___SETFPUCW */
-
 /* Define to disable Valgrind instrumentation */
 #define NVALGRIND 1
 
 /* Define if using GNU-style Objective C runtime. */
-#define OBJC_GNU_RUNTIME 1
+/* #undef OBJC_GNU_RUNTIME */
 
 /* Define if using NeXT/Apple-style Objective C runtime. */
 /* #undef OBJC_NEXT_RUNTIME */
@@ -967,11 +1093,14 @@
 #define PACKAGE_TARNAME "R"
 
 /* Define to the home page for this package. */
-#define PACKAGE_URL "http://www.r-project.org"
+#define PACKAGE_URL "https://www.r-project.org"
 
 /* Define to the version of this package. */
 #define PACKAGE_VERSION "@VERSION@"
 
+/* PCRE2 code unit width wanted. */
+#define PCRE2_CODE_UNIT_WIDTH 8
+
 /* Define if <inttypes.h> exists and defines unusable PRI* macros. (For intl)
    */
 /* #undef PRI_MACROS_BROKEN */
@@ -979,19 +1108,20 @@
 /* Define if the pthread_in_use() detection is hard. (For intl) */
 /* #undef PTHREAD_IN_USE_DETECTION_HARD */
 
-/* Define as the return type of signal handlers (`int' or `void'). */
-#define RETSIGTYPE void
-
 /* Define this to use architecture-dependent subdirectories of this name. */
 #ifndef R_ARCH
 #define R_ARCH ""
 #endif
 
 /* Define this to be the name of the CPU of your system. */
-#ifdef _WIN64
-#define R_CPU "x86_64"
+#ifdef __aarch64__
+# define R_CPU "aarch64"
 #else
-#define R_CPU "i386"
+# ifdef _WIN64
+#  define R_CPU "x86_64"
+# else
+#  define R_CPU "i386"
+# endif
 #endif
 
 /* Define as `inline', or `__inline__' or `__inline' if that's what the C
@@ -1007,10 +1137,22 @@
 #define R_OS "mingw32"
 
 /* Define this to be the canonical name (cpu-vendor-os) of your system. */
-#ifdef _WIN64
-#define R_PLATFORM "x86_64-w64-mingw32"
+#ifdef __aarch64__
+# define R_PLATFORM "aarch64-w64-mingw32"
 #else
-#define R_PLATFORM "i386-w64-mingw32"
+# ifdef _WIN64
+#  define R_PLATFORM "x86_64-w64-mingw32"
+# else
+#  define R_PLATFORM "i386-w64-mingw32"
+# endif
+#endif
+
+/* Define this to be the C runtime R has been built for. */
+#if defined(_UCRT) || ((__MSVCRT_VERSION__ >= 0x1400) || (__MSVCRT_VERSION__ >= 0xE00 && __MSVCRT_VERSION__ < 0x1000))
+  /* __MSVCRT_VERSION__ is defined in _mingw.h. Keep in step with corecrt.h. */
+  #define R_CRT "ucrt"
+#else
+  #define R_CRT "msvcrt"
 #endif
 
 /* Define this to be printing command on your system. */
@@ -1038,18 +1180,31 @@
 #define SIZEOF_LONG 4
 
 /* The size of `long double', as computed by sizeof. */
-#ifdef _WIN64
-#define SIZEOF_LONG_DOUBLE 16
+#ifdef __aarch64__
+# define SIZEOF_LONG_DOUBLE 8
 #else
-#define SIZEOF_LONG_DOUBLE 12
+# ifdef _WIN64
+#  define SIZEOF_LONG_DOUBLE 16
+# else
+#  define SIZEOF_LONG_DOUBLE 12
+# endif
 #endif
 
 /* The size of `long long', as computed by sizeof. */
 #define SIZEOF_LONG_LONG 8
 
 /* The size of `size_t', as computed by sizeof. */
-#define SIZEOF_SIZE_T @ST@
+#ifdef _WIN64
+# define SIZEOF_SIZE_T 8
+#else
+# define SIZEOF_SIZE_T 4
+#endif
 
+/* Workaround for win64 pow() precision issue in Mingw-w64 V3 and higher
+   See http://sourceforge.net/p/mingw-w64/bugs/466 for discussion. */
+#if defined(_WIN64) && defined(__MINGW64_VERSION_MAJOR) && __MINGW64_VERSION_MAJOR >= 3
+#define USE_POWL_IN_R_POW 1
+#endif
 
 /* Define as the maximum value of type 'size_t', if the system doesn't define
    it. (For intl) */
@@ -1063,12 +1218,11 @@
 	STACK_DIRECTION = 0 => direction of growth unknown */
 /* #undef STACK_DIRECTION */
 
-/* Define to 1 if you have the ANSI C header files. */
+/* Define to 1 if all of the C90 standard headers exist (not just the ones
+   required in a freestanding environment). This macro is provided for
+   backward compatibility; new code need not use it. */
 #define STDC_HEADERS 1
 
-/* Define if you provide support for the libxml ftp/http functions. */
-#define SUPPORT_LIBXML 1
-
 /* Define if you have C/C++/Fortran OpenMP support for package code. */
 /* #undef SUPPORT_OPENMP */
 
@@ -1088,6 +1242,10 @@
 /* Define to use internal time-zone code */
 #define USE_INTERNAL_MKTIME 1
 
+/* Define to use libdefault rather than libz for lazy-loaded R objects */
+/* #undef USE_LIBDEFLATE */
+#define USE_LIBDEFLATE 1
+
 /* Define if the POSIX multithreading library can be used. (For intl) */
 /* #undef USE_POSIX_THREADS */
 
@@ -1102,6 +1260,15 @@
    weak. (For intl) */
 /* #undef USE_PTH_THREADS_WEAK */
 
+/* Define to 1 to use internal code for 'towlower' and 'towupper'. */
+#define USE_RI18N_CASE 1
+
+/* Define to 1 to use internal 'iswprint' etc. */
+#define USE_RI18N_FNS 1
+
+/* Define to 1 to use internal 'wcwidth' */
+#define USE_RI18N_WIDTH
+
 /* Define if the old Solaris multithreading library can be used. (For intl) */
 /* #undef USE_SOLARIS_THREADS */
 
@@ -1174,7 +1341,7 @@
    this defined. */
 /* #undef _POSIX_1_SOURCE */
 
-/* Define to 1 if you need to in order for `stat' and other things to work. */
+/* Define to 1 if you need to in order for 'stat' and other things to work. */
 /* #undef _POSIX_SOURCE */
 
 /* Define for Solaris 2.5.1 so the uint64_t typedef from <sys/synch.h>,
@@ -1195,7 +1362,7 @@
 /* #undef inline */
 #endif
 
-/* Define to `int' if <sys/types.h> does not define. */
+/* Define as a signed integer type capable of holding a process identifier. */
 /* #undef pid_t */
 
 /* Define as the type of the result of subtracting two pointers, if the system
