@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998--2022  The R Core Team.
+ *  Copyright (C) 1998--2024  The R Core Team.
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,13 @@
 
 #define BYTECODE
 
+#if ( SIZEOF_SIZE_T < SIZEOF_DOUBLE )
+# define BOXED_BINDING_CELLS 1
+#else
+# define BOXED_BINDING_CELLS 0
+# define IMMEDIATE_PROMISE_VALUES
+#endif
+
 /* probably no longer needed */
 #define NEW_CONDITION_HANDLING
 
@@ -58,10 +65,25 @@
 # define extern0 extern
 #endif
 
+#define attribute_no_sanitizer_instrumentation
+#ifdef __has_attribute
+# if __has_attribute(disable_sanitizer_instrumentation)
+#  undef attribute_no_sanitizer_instrumentation
+#  define attribute_no_sanitizer_instrumentation \
+     __attribute__((disable_sanitizer_instrumentation))
+# elif __has_attribute(no_sanitize)
+#  undef attribute_no_sanitizer_instrumentation
+#  define attribute_no_sanitizer_instrumentation \
+     __attribute__ ((no_sanitize ("address", "thread", "leak", "undefined")))
+# endif
+#endif
+
 #define MAXELTSIZE 8192 /* Used as a default for string buffer sizes,
 			   and occasionally as a limit. */
 
 #include <R_ext/Complex.h>
+#include <R_ext/Print.h>
+
 void Rf_CoercionWarning(int);/* warning code */
 int Rf_LogicalFromInteger(int, int*);
 int Rf_LogicalFromReal(double, int*);
@@ -296,7 +318,7 @@ typedef union { VECTOR_SEXPREC s; double align; } SEXPREC_ALIGN;
 /* The same bit can be used to mark calls used in complex assignments
    to allow replacement functions to determine when they are being
    called in an assignment context and can modify an object with one
-   refrence */
+   reference */
 #define MARK_ASSIGNMENT_CALL(call) SET_ASSIGNMENT_PENDING(call, TRUE)
 #define IS_ASSIGNMENT_CALL(call) ASSIGNMENT_PENDING(call)
 
@@ -395,7 +417,6 @@ typedef union { VECTOR_SEXPREC s; double align; } SEXPREC_ALIGN;
 #define RAW(x)		((Rbyte *) DATAPTR(x))
 #define COMPLEX(x)	((Rcomplex *) DATAPTR(x))
 #define REAL(x)		((double *) DATAPTR(x))
-#define VECTOR_ELT(x,i)	((SEXP *) DATAPTR(x))[i]
 #define STRING_PTR(x)	((SEXP *) DATAPTR(x))
 #define VECTOR_PTR(x)	((SEXP *) DATAPTR(x))
 #define LOGICAL_RO(x)	((const int *) DATAPTR_RO(x))
@@ -422,6 +443,7 @@ typedef union { VECTOR_SEXPREC s; double align; } SEXPREC_ALIGN;
 #define CADDR(e)	CAR(CDR(CDR(e)))
 #define CADDDR(e)	CAR(CDR(CDR(CDR(e))))
 #define CAD4R(e)	CAR(CDR(CDR(CDR(CDR(e)))))
+#define CAD5R(e)	CAR(CDR(CDR(CDR(CDR(CDR(e))))))
 #define MISSING_MASK	15 /* reserve 4 bits--only 2 uses now */
 #define MISSING(x)	((x)->sxpinfo.gp & MISSING_MASK)/* for closure calls */
 #define SET_MISSING(x,v) do { \
@@ -433,11 +455,6 @@ typedef union { VECTOR_SEXPREC s; double align; } SEXPREC_ALIGN;
 #define BNDCELL_TAG(e)	((e)->sxpinfo.extra)
 #define SET_BNDCELL_TAG(e, v) ((e)->sxpinfo.extra = (v))
 
-#if ( SIZEOF_SIZE_T < SIZEOF_DOUBLE )
-# define BOXED_BINDING_CELLS 1
-#else
-# define BOXED_BINDING_CELLS 0
-#endif
 #if BOXED_BINDING_CELLS
 /* Use allocated scalars to hold immediate binding values. A little
    less efficient but does not change memory layout or use. These
@@ -529,7 +546,7 @@ typedef union {
 #define IS_GETTER_CALL(call) (CADR(call) == R_TmpvalSymbol)
 
 #ifdef LONG_VECTOR_SUPPORT
-    R_len_t NORET R_BadLongVector(SEXP, const char *, int);
+    NORET R_len_t R_BadLongVector(SEXP, const char *, int);
 #endif
 
 /* checking for mis-use of multi-threading */
@@ -604,6 +621,8 @@ void (SET_BNDCELL_IVAL)(SEXP cell, int v);
 void (SET_BNDCELL_LVAL)(SEXP cell, int v);
 void (INIT_BNDCELL)(SEXP cell, int type);
 void SET_BNDCELL(SEXP cell, SEXP val);
+int (PROMISE_TAG)(SEXP e);
+void (SET_PROMISE_TAG)(SEXP e, int v);
 
 /* List Access Functions */
 SEXP (CAR0)(SEXP e);
@@ -627,6 +646,8 @@ void (SET_PRSEEN)(SEXP x, int v);
 void SET_PRENV(SEXP x, SEXP v);
 void SET_PRVALUE(SEXP x, SEXP v);
 void SET_PRCODE(SEXP x, SEXP v);
+void IF_PROMSXP_SET_PRVALUE(SEXP x, SEXP v);
+int  (PROMISE_IS_EVALUATED)(SEXP x);
 
 /* Hashing Functions */
 int  (HASHASH)(SEXP x);
@@ -668,6 +689,8 @@ Rcomplex ALTCOMPLEX_ELT(SEXP x, R_xlen_t i);
 void ALTCOMPLEX_SET_ELT(SEXP x, R_xlen_t i, Rcomplex v);
 Rbyte ALTRAW_ELT(SEXP x, R_xlen_t i);
 void ALTRAW_SET_ELT(SEXP x, R_xlen_t i, Rbyte v);
+SEXP ALTLIST_ELT(SEXP, R_xlen_t);
+void ALTLIST_SET_ELT(SEXP, R_xlen_t, SEXP);
 
 /* invoking ALTREP class methods */
 SEXP ALTINTEGER_SUM(SEXP x, Rboolean narm);
@@ -694,14 +717,14 @@ Rboolean Rf_psmatch(const char *, const char *, Rboolean);
 void Rf_printwhere(void);
 void Rf_readS3VarsFromFrame(SEXP, SEXP*, SEXP*, SEXP*, SEXP*, SEXP*, SEXP*);
 
-void NORET R_signal_protect_error(void);
-void NORET R_signal_unprotect_error(void);
-void NORET R_signal_reprotect_error(PROTECT_INDEX i);
+NORET void R_signal_protect_error(void);
+NORET void R_signal_unprotect_error(void);
+NORET void R_signal_reprotect_error(PROTECT_INDEX i);
 
-const char *R_curErrorBuf();
+const char *R_curErrorBuf(void);
 Rboolean R_cycle_detected(SEXP s, SEXP child);
 
-void R_init_altrep();
+void R_init_altrep(void);
 void R_reinit_altrep_classes(DllInfo *);
 
 /* Defining NO_RINLINEDFUNS disables use to simulate platforms where
@@ -752,21 +775,32 @@ void SET_SCALAR_BVAL(SEXP x, Rbyte v);
 
 /* macro version of R_CheckStack */
 #define R_CheckStack() do {						\
-	void NORET R_SignalCStackOverflow(intptr_t);				\
+	NORET void R_SignalCStackOverflow(intptr_t);				\
 	int dummy;							\
 	intptr_t usage = R_CStackDir * (R_CStackStart - (uintptr_t)&dummy); \
 	if(R_CStackLimit != (uintptr_t)(-1) && usage > ((intptr_t) R_CStackLimit)) \
 	    R_SignalCStackOverflow(usage);				\
     } while (FALSE)
-#endif /* USE_RINTERNALS */
 
-void R_BadValueInRCode(SEXP value, SEXP call, SEXP rho, const char *rawmsg,
-        const char *errmsg, const char *warnmsg, const char *varname,
-        Rboolean warnByDefault);
+#ifdef __has_feature
+# if __has_feature(address_sanitizer)
+#  undef R_CheckStack
+# endif
+#endif
+
+#ifdef R_CheckStack
+# if defined(__SANITIZE_ADDRESS__) || defined(__SANITIZE_THREAD__)
+#  undef R_CheckStack
+# endif
+#endif
+
+#endif /* USE_RINTERNALS */
 
 const char * Rf_translateCharFP(SEXP);
 const char * Rf_translateCharFP2(SEXP);
 const char * Rf_trCharUTF8(SEXP);
+const char * Rf_trCharUTF82(SEXP);
+const wchar_t * Rf_wtransChar2(SEXP);
 
 extern0 SEXP	R_CommentSymbol;    /* "comment" */
 extern0 SEXP	R_DotEnvSymbol;     /* ".Environment" */
@@ -947,17 +981,22 @@ extern char *Rstrdup(const char *s);
 extern int putenv(char *string);
 #endif
 
+/* PATH_MAX has historically been understood to be the maximal length in
+   bytes of an entire path name that may exist.  In current POSIX, when
+   defined, it is the maximum number of bytes that will be stored to a
+   user-supplied buffer by file-system operations which do not allow the
+   caller to provide actual size of the buffer (and such calls are rare).
+   If the system limited path names to a certain value, it shall not be less
+   than this value, if defined.
 
-/* Maximal length in bytes of an entire path name.
    POSIX has required this to be at least 255/256, and X/Open at least 1024.
-   Solaris, macOS, *BSD have 1024, Linux glibc has 4192.
-   File names are limited to FILENAME_MAX bytes (usually the same as PATH_MAX)
-   or NAME_MAX (often 255/256).
+   Solaris, macOS, *BSD have 1024, Linux glibc has 4192 (but glibc does not
+   enforce the limit).  File names are limited to FILENAME_MAX bytes
+   (usually the same as PATH_MAX) or NAME_MAX (often 255/256).
 
-   POSIX requires PATH_MAX to be defined in limits.h (included above)
-   if independent of the file path (file system?).  However, if it can
-   vary by filepath, it is required to be undefined there.
- */
+   POSIX requires PATH_MAX to be defined in limits.h (included above) if
+   independent of the file path (file system).  However, if it can vary by
+   filepath, it is required to be undefined there. */
 #if !defined(PATH_MAX)
 # if defined(HAVE_SYS_PARAM_H)
 #  include <sys/param.h>
@@ -967,12 +1006,25 @@ extern int putenv(char *string);
 /* Try BSD name */
 #    define PATH_MAX MAXPATHLEN
 #  elif defined(Win32)
-/* seems this is now defined by MinGW to be 259, whereas FILENAME_MAX
-   and MAX_PATH are 260.  It is not clear that this really is in bytes,
-   but might be chars for the Unicode interfaces.
+/* MinGW-W64 defines PATH_MAX to MAX_PATH, which is 260 in Windows. It used to
+   be the maximal length of an entire path name (depending on the API used,
+   in bytes or in UCS-2 units, including the terminator).  This is no longer
+   the case and like PATH_MAX on Unix, MAX_PATH is only used as the limit in
+   several old API calls for the user buffer when the size cannot be passed
+   by user.  Some additional old components of Windows and old APIs seem not
+   to support longer paths.
 
-   260 is d:\ plus 256 chars plus nul.  Some but not all API calls
-   allow filepaths of the form \\?\D:\very_long_path .
+   But longer paths can and do exist, they can be created using some API,
+   usually UCS-2, sometimes using the extended syntax
+   (\\?\D:\very_long_path) but sometimes using the common syntax.
+   Applications that haven't opted for "long paths" (and those also have to
+   be enabled system-wide) are shielded from seeing the long paths in some
+   API calls to prevent buffer-overflow and other errors in user code that
+   did not check the lengths.
+
+   Like on Unix, user/library code should not depend on that there is a
+   maximum to the path length.  The actual real maximum on Windows is not
+   even precisely specified.
 */
 #    define PATH_MAX 260
 #  else
@@ -982,6 +1034,18 @@ extern int putenv(char *string);
 # endif
 #endif
 
+/* R_PATH_MAX is an R-defined limit on the length of entire path name,
+   including the terminator.  It may be used in some older code that has not
+   yet been rewritten to support arbitrary path lengths.  Such code still
+   has to detect longer paths, but may not support them, e.g.  may throw an
+   error. R_PATH_MAX shall be at least PATH_MAX, when PATH_MAX is defined. */
+#ifdef Unix
+# define R_PATH_MAX PATH_MAX
+#else
+  /* On Windows, 260 is too limiting */
+# define R_PATH_MAX 5000
+#endif
+
 #ifdef R_USE_SIGNALS
 #ifdef HAVE_POSIX_SETJMP
 # define SIGJMP_BUF sigjmp_buf
@@ -1092,9 +1156,22 @@ typedef struct {
 /* Promise Access Macros */
 #define PRCODE(x)	((x)->u.promsxp.expr)
 #define PRENV(x)	((x)->u.promsxp.env)
-#define PRVALUE(x)	((x)->u.promsxp.value)
 #define PRSEEN(x)	((x)->sxpinfo.gp)
 #define SET_PRSEEN(x,v)	(((x)->sxpinfo.gp)=(v))
+#ifdef IMMEDIATE_PROMISE_VALUES
+# define PRVALUE0(x) ((x)->u.promsxp.value)
+# define PRVALUE(x) \
+    (PROMISE_TAG(x) ? R_expand_promise_value(x) : PRVALUE0(x))
+# define PROMISE_IS_EVALUATED(x) \
+    (PROMISE_TAG(x) || PRVALUE0(x) != R_UnboundValue)
+# define PROMISE_TAG(x)  BNDCELL_TAG(x)
+# define SET_PROMISE_TAG(x, v) SET_BNDCELL_TAG(x, v)
+#else
+# define PRVALUE0(x) ((x)->u.promsxp.value)
+# define PRVALUE(x) PRVALUE0(x)
+# define PROMISE_IS_EVALUATED(x) (PRVALUE(x) != R_UnboundValue)
+# define PROMISE_TAG(x) 0
+#endif
 
 /* Hashing Macros */
 #define HASHASH(x)      ((x)->sxpinfo.gp & HASHASH_MASK)
@@ -1200,7 +1277,7 @@ Rboolean (NO_SPECIAL_SYMBOLS)(SEXP b);
    compiled 'for' loops. This could be used more extensively in the
    future, though the ALTREP framework may be a better choice.
 
-   Allocating on the stack memory is also supported; this is currently
+   Allocating memory on the stack is also supported; this is currently
    used for jump buffers.
 */
 typedef struct {
@@ -1217,6 +1294,9 @@ typedef struct {
 # define RAWMEM_TAG 254
 # define CACHESZ_TAG 253
 
+/* saved bcEval() state for implementing recursion using goto */
+typedef struct R_bcFrame R_bcFrame_type;
+
 #ifdef R_USE_SIGNALS
 /* Stack entry for pending promises */
 typedef struct RPRSTACK {
@@ -1245,15 +1325,17 @@ typedef struct RCNTXT {
     int bcintactive;            /* R_BCIntActive value */
     SEXP bcbody;                /* R_BCbody value */
     void* bcpc;                 /* R_BCpc value */
+    ptrdiff_t relpc;            /* pc offset when begincontext is called */
     SEXP handlerstack;          /* condition handler stack */
     SEXP restartstack;          /* stack of available restarts */
     struct RPRSTACK *prstack;   /* stack of pending promises */
     R_bcstack_t *nodestack;
     R_bcstack_t *bcprottop;
+    R_bcFrame_type *bcframe;
     SEXP srcref;	        /* The source line in effect */
     int browserfinish;          /* should browser finish this context without
                                    stopping */
-    SEXP returnValue;           /* only set during on.exit calls */
+    R_bcstack_t returnValue;    /* only set during on.exit calls */
     struct RCNTXT *jumptarget;	/* target for a continuing jump */
     int jumpmask;               /* associated LONGJMP argument */
 } RCNTXT, *context;
@@ -1283,6 +1365,9 @@ enum {
     CTXT_UNWIND   = 128
 };
 
+extern0 RCNTXT *getLexicalContext(SEXP);
+extern0 SEXP getLexicalCall(SEXP);
+
 /*
 TOP   0 0 0 0 0 0  = 0
 NEX   1 0 0 0 0 0  = 1
@@ -1377,6 +1462,7 @@ extern0 int	R_BCIntActive INI_as(0); /* bcEval called more recently than
                                             eval */
 extern0 void*	R_BCpc INI_as(NULL);/* current byte code instruction */
 extern0 SEXP	R_BCbody INI_as(NULL); /* current byte code object */
+extern0 R_bcFrame_type *R_BCFrame INI_as(NULL); /* bcEval() frame */
 extern0 SEXP	R_NHeap;	    /* Start of the cons cell heap */
 extern0 SEXP	R_FreeSEXP;	    /* Cons cell free list */
 extern0 R_size_t R_Collected;	    /* Number of free cons cells (after gc) */
@@ -1505,7 +1591,7 @@ extern0 double elapsedLimitValue       	INI_as(-1.0);
 void resetTimeLimits(void);
 void R_CheckTimeLimits(void);
 
-#define R_BCNODESTACKSIZE 200000
+#define R_BCNODESTACKSIZE 300000
 LibExtern R_bcstack_t *R_BCNodeStackTop, *R_BCNodeStackEnd;
 extern0 R_bcstack_t *R_BCNodeStackBase;
 extern0 R_bcstack_t *R_BCProtTop;
@@ -1515,12 +1601,13 @@ extern0 int R_check_constants INI_as(0);
 extern0 int R_disable_bytecode INI_as(0);
 extern SEXP R_cmpfun1(SEXP); /* unconditional fresh compilation */
 extern void R_init_jit_enabled(void);
-extern void R_initAssignSymbols(void);
+extern void R_initEvalSymbols(void);
 #ifdef R_USE_SIGNALS
 extern SEXP R_findBCInterpreterSrcref(RCNTXT*);
 #endif
-extern SEXP R_getCurrentSrcref();
-extern SEXP R_getBCInterpreterExpression();
+extern SEXP R_getCurrentSrcref(void);
+extern SEXP R_getBCInterpreterExpression(void);
+extern ptrdiff_t R_BCRelPC(SEXP, void *);
 
 void R_BCProtReset(R_bcstack_t *);
 
@@ -1585,18 +1672,20 @@ extern0 int R_PCRE_limit_recursion;
 /*--- FUNCTIONS ------------------------------------------------------ */
 
 /* Internal type coercions */
-int Rf_asLogical2(SEXP x, int checking, SEXP call, SEXP rho);
+int Rf_asLogical2(SEXP x, int checking, SEXP call);
 
 
 typedef enum { iSILENT, iWARN, iERROR } warn_type;
 
 /* Other Internally Used Functions, excluding those which are inline-able*/
+SEXP Rf_applyClosure(SEXP, SEXP, SEXP, SEXP, SEXP, Rboolean);
 void Rf_addMissingVarsToNewEnv(SEXP, SEXP);
 SEXP Rf_allocFormalsList2(SEXP sym1, SEXP sym2);
 SEXP Rf_allocFormalsList3(SEXP sym1, SEXP sym2, SEXP sym3);
 SEXP Rf_allocFormalsList4(SEXP sym1, SEXP sym2, SEXP sym3, SEXP sym4);
 SEXP Rf_allocFormalsList5(SEXP sym1, SEXP sym2, SEXP sym3, SEXP sym4, SEXP sym5);
 SEXP Rf_allocFormalsList6(SEXP sym1, SEXP sym2, SEXP sym3, SEXP sym4, SEXP sym5, SEXP sym6);
+SEXP R_allocObject(void);
 SEXP Rf_arraySubscript(int, SEXP, SEXP, SEXP (*)(SEXP,SEXP),
                        SEXP (*)(SEXP, int), SEXP);
 SEXP Rf_fixSubset3Args(SEXP, SEXP, SEXP, SEXP*);
@@ -1721,6 +1810,7 @@ void R_RestoreHashCount(SEXP rho);
 # define mbtoucs		Rf_mbtoucs
 # define mbcsToUcs2		Rf_mbcsToUcs2
 # define memtrace_report	Rf_memtrace_report
+# define mkCharWUTF8		Rf_mkCharWUTF8
 # define mkCLOSXP		Rf_mkCLOSXP
 # define mkFalse		Rf_mkFalse
 # define mkPROMISE		Rf_mkPROMISE
@@ -1767,15 +1857,13 @@ void R_RestoreHashCount(SEXP rho);
 # define translateCharFP	Rf_translateCharFP
 # define translateCharFP2	Rf_translateCharFP2
 # define trCharUTF8      	Rf_trCharUTF8
+# define trCharUTF82      	Rf_trCharUTF82
 # define tspgets		Rf_tspgets
 # define type2symbol		Rf_type2symbol
 # define unbindVar		Rf_unbindVar
 # define usemethod		Rf_usemethod
 # define ucstomb		Rf_ucstomb
 # define ucstoutf8		Rf_ucstoutf8
-#ifdef ADJUST_ENVIR_REFCNTS
-# define unpromiseArgs		Rf_unpromiseArgs
-#endif
 # define utf8toucs		Rf_utf8toucs
 # define utf8towcs		Rf_utf8towcs
 # define vectorIndex		Rf_vectorIndex
@@ -1783,6 +1871,7 @@ void R_RestoreHashCount(SEXP rho);
 # define WarningMessage		Rf_WarningMessage
 # define wcstoutf8		Rf_wcstoutf8
 # define wtransChar		Rf_wtransChar
+# define wtransChar2		Rf_wtransChar2
 # define yychar			Rf_yychar
 # define yylval			Rf_yylval
 # define yynerrs		Rf_yynerrs
@@ -1812,8 +1901,9 @@ char	*R_HomeDir(void);
 Rboolean R_FileExists(const char *);
 Rboolean R_HiddenFile(const char *);
 double	R_FileMtime(const char *);
-int	R_GetFDLimit();
+int	R_GetFDLimit(void);
 int	R_EnsureFDLimit(int);
+Rboolean R_IsDirPath(const char *);
 
 /* environment cell access */
 typedef struct { SEXP cell; } R_varloc_t; /* use struct to prevent casting */
@@ -1873,6 +1963,30 @@ typedef struct {
     SEXP callArgs;
 } R_PrintData;
 
+/* Dirent wrappers/implementation */
+
+struct R_dirent {
+    char *d_name; /* null-terminated filename */
+};
+
+typedef struct R_DIR_INTERNAL R_DIR;
+
+R_DIR *R_opendir(const char *name);
+struct R_dirent *R_readdir(R_DIR *rdir);
+int R_closedir(R_DIR *rdir);
+
+#ifdef Win32
+struct R_wdirent {
+    wchar_t *d_name; /* null-terminated filename */
+};
+
+typedef struct R_WDIR_INTERNAL R_WDIR;
+
+R_WDIR *R_wopendir(const wchar_t *name);
+struct R_wdirent *R_wreaddir(R_WDIR *rdir);
+int R_wclosedir(R_WDIR *rdir);
+#endif
+
 /* Other Internally Used Functions */
 
 SEXP Rf_allocCharsxp(R_len_t);
@@ -1881,7 +1995,7 @@ R_xlen_t asVecSize(SEXP x);
 R_xlen_t asXLength(SEXP x);
 void check1arg(SEXP, SEXP, const char *);
 void Rf_checkArityCall(SEXP, SEXP, SEXP);
-void CheckFormals(SEXP);
+void CheckFormals(SEXP, const char*);
 void R_check_locale(void);
 void check_stack_balance(SEXP op, int save);
 void CleanEd(void);
@@ -1909,7 +2023,7 @@ R_xlen_t any_duplicated3(SEXP, SEXP, Rboolean);
 SEXP evalList(SEXP, SEXP, SEXP, int);
 SEXP evalListKeepMissing(SEXP, SEXP);
 int factorsConform(SEXP, SEXP);
-void NORET findcontext(int, SEXP, SEXP);
+NORET void findcontext(int, SEXP, SEXP);
 SEXP findVar1(SEXP, SEXP, SEXPTYPE, int);
 void FrameClassFix(SEXP);
 SEXP frameSubscript(int, SEXP, SEXP);
@@ -1943,7 +2057,7 @@ void InitS3DefaultTypes(void);
 void internalTypeCheck(SEXP, SEXP, SEXPTYPE);
 Rboolean isMethodsDispatchOn(void);
 int isValidName(const char *);
-void NORET jump_to_toplevel(void);
+NORET void jump_to_toplevel(void);
 void KillAllDevices(void);
 SEXP levelsgets(SEXP, SEXP);
 void mainloop(void);
@@ -1956,6 +2070,7 @@ SEXP matchArgs_NR(SEXP, SEXP, SEXP);
 SEXP matchArgs_RC(SEXP, SEXP, SEXP);
 SEXP matchPar(const char *, SEXP*);
 void memtrace_report(void *, void *);
+SEXP mkCharWUTF8(const wchar_t *);
 SEXP mkCLOSXP(SEXP, SEXP, SEXP);
 SEXP mkFalse(void);
 SEXP mkPRIMSXP (int, int);
@@ -1987,12 +2102,13 @@ void process_site_Renviron(void);
 void process_system_Renviron(void);
 void process_user_Renviron(void);
 SEXP promiseArgs(SEXP, SEXP);
-void Rcons_vprintf(const char *, va_list);
+int Rcons_vprintf(const char *, va_list);
+int REvprintf_internal(const char *, va_list);
 SEXP R_data_class(SEXP , Rboolean);
 SEXP R_data_class2(SEXP);
 char *R_LibraryFileName(const char *, char *, size_t);
 SEXP R_LoadFromFile(FILE*, int);
-SEXP R_NewHashedEnv(SEXP, SEXP);
+SEXP R_NewHashedEnv(SEXP, int);
 extern int R_Newhashpjw(const char *);
 FILE* R_OpenLibraryFile(const char *);
 SEXP R_Primitive(const char *);
@@ -2005,9 +2121,12 @@ void R_SaveToFileV(SEXP, FILE*, int, int);
 Rboolean R_seemsOldStyleS4Object(SEXP object);
 int R_SetOptionWarn(int);
 int R_SetOptionWidth(int);
+SEXP R_SetOption(SEXP, SEXP);
 void R_Suicide(const char *);
+SEXP R_flexiblas_info(void);
 void R_getProcTime(double *data);
-int R_isMissing(SEXP symbol, SEXP rho);
+Rboolean R_isMissing(SEXP symbol, SEXP rho);
+Rboolean R_missing(SEXP symbol, SEXP rho);
 const char *sexptype2char(SEXPTYPE type);
 void sortVector(SEXP, Rboolean);
 void SrcrefPrompt(const char *, SEXP);
@@ -2022,9 +2141,6 @@ void unbindVar(SEXP, SEXP);
 #ifdef ALLOW_OLD_SAVE
 void unmarkPhase(void);
 #endif
-#ifdef ADJUST_ENVIR_REFCNTS
-void unpromiseArgs(SEXP);
-#endif
 SEXP R_LookupMethod(SEXP, SEXP, SEXP, SEXP);
 int usemethod(const char *, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP*);
 SEXP vectorIndex(SEXP, SEXP, int, int, int, SEXP, Rboolean);
@@ -2035,7 +2151,7 @@ SEXP dynamicfindVar(SEXP, RCNTXT*);
 void endcontext(RCNTXT*);
 int framedepth(RCNTXT*);
 void R_InsertRestartHandlers(RCNTXT *, const char *);
-void NORET R_JumpToContext(RCNTXT *, int, SEXP);
+NORET void R_JumpToContext(RCNTXT *, int, SEXP);
 SEXP R_syscall(int,RCNTXT*);
 int R_sysparent(int,RCNTXT*);
 SEXP R_sysframe(int,RCNTXT*);
@@ -2044,44 +2160,54 @@ RCNTXT *R_findExecContext(RCNTXT *, SEXP);
 RCNTXT *R_findParentContext(RCNTXT *, int);
 
 void R_run_onexits(RCNTXT *);
-void NORET R_jumpctxt(RCNTXT *, int, SEXP);
+NORET void R_jumpctxt(RCNTXT *, int, SEXP);
 #endif
 
 /* ../main/bind.c */
 SEXP ItemName(SEXP, R_xlen_t);
 
 /* ../main/errors.c : */
-void NORET errorcall_cpy(SEXP, const char *, ...);
-void NORET ErrorMessage(SEXP, int, ...);
+NORET void errorcall_cpy(SEXP, const char *, ...) R_PRINTF_FORMAT(2,3);
+NORET void ErrorMessage(SEXP, int, ...);
 void WarningMessage(SEXP, R_WARNING, ...);
 SEXP R_GetTraceback(int);    // including deparse()ing
 SEXP R_GetTracebackOnly(int);// no        deparse()ing
-void NORET R_signalErrorCondition(SEXP cond, SEXP call);
-void NORET R_signalErrorConditionEx(SEXP cond, SEXP call, int exitOnly);
+NORET void R_signalErrorCondition(SEXP cond, SEXP call);
+NORET void R_signalErrorConditionEx(SEXP cond, SEXP call, int exitOnly);
 SEXP R_vmakeErrorCondition(SEXP call,
 			   const char *classname, const char *subclassname,
-			   int nextra, const char *format, va_list ap);
+			   int nextra, const char *format, va_list ap)
+     R_PRINTF_FORMAT(5,0);
+
 SEXP R_makeErrorCondition(SEXP call,
 			  const char *classname, const char *subclassname,
-			  int nextra, const char *format, ...);
+			  int nextra, const char *format, ...)
+     R_PRINTF_FORMAT(5,0);
+
 void R_setConditionField(SEXP cond, R_xlen_t idx, const char *name, SEXP val);
 SEXP R_makeNotSubsettableError(SEXP x, SEXP call);
+SEXP R_makeMissingSubscriptError(SEXP x, SEXP call);
+SEXP R_makeMissingSubscriptError1(SEXP call);
 SEXP R_makeOutOfBoundsError(SEXP x, int subscript, SEXP sindex,
 			    SEXP call, const char *prefix);
 SEXP R_makeCStackOverflowError(SEXP call, intptr_t usage);
-SEXP R_getProtectStackOverflowError();
-SEXP R_getExpressionStackOverflowError();
-SEXP R_getNodeStackOverflowError();
-void R_InitConditions();
+SEXP R_getProtectStackOverflowError(void);
+SEXP R_getExpressionStackOverflowError(void);
+SEXP R_getNodeStackOverflowError(void);
+void R_InitConditions(void);
 
 R_size_t R_GetMaxVSize(void);
-void R_SetMaxVSize(R_size_t);
+Rboolean R_SetMaxVSize(R_size_t);
 R_size_t R_GetMaxNSize(void);
-void R_SetMaxNSize(R_size_t);
+Rboolean R_SetMaxNSize(R_size_t);
 R_size_t R_Decode2Long(char *p, int *ierr);
 void R_SetPPSize(R_size_t);
+void R_SetNconn(int);
 
 void R_expand_binding_value(SEXP);
+#ifdef IMMEDIATE_PROMISE_VALUES
+SEXP R_expand_promise_value(SEXP);
+#endif
 
 void R_args_enable_refcnt(SEXP);
 void R_try_clear_args_refcnt(SEXP);
@@ -2089,6 +2215,10 @@ void R_try_clear_args_refcnt(SEXP);
 /* ../main/devices.c, used in memory.c, gnuwin32/extra.c */
 #define R_MaxDevices 64
 
+/* gnuwin32/extra.c */
+wchar_t *R_getFullPathNameW(const wchar_t *);
+char *R_getFullPathName(const char *);
+
 /* ../../main/printutils.c : */
 typedef enum {
     Rprt_adj_left = 0,
@@ -2119,8 +2249,8 @@ SEXP R_subassign3_dflt(SEXP, SEXP, SEXP, SEXP);
 #include <wchar.h>
 
 /* main/util.c */
-void NORET UNIMPLEMENTED_TYPE(const char *s, SEXP x);
-void NORET UNIMPLEMENTED_TYPEt(const char *s, SEXPTYPE t);
+NORET void UNIMPLEMENTED_TYPE(const char *s, SEXP x);
+NORET void UNIMPLEMENTED_TYPEt(const char *s, SEXPTYPE t);
 Rboolean Rf_strIsASCII(const char *str);
 int utf8clen(char c);
 int Rf_AdobeSymbol2ucs2(int n);
@@ -2151,13 +2281,19 @@ char *mbcsTruncateToValid(char *s);
 Rboolean utf8Valid(const char *str);
 char *Rf_strchr(const char *s, int c);
 char *Rf_strrchr(const char *s, int c);
-int Rvsnprintf_mbcs(char *buf, size_t size, const char *format, va_list ap);
-int Rsnprintf_mbcs(char *str, size_t size, const char *format, ...);
+int Rvsnprintf_mbcs(char *buf, size_t size, const char *format, va_list ap)
+    R_PRINTF_FORMAT(3,0);
+
+int Rsnprintf_mbcs(char *str, size_t size, const char *format, ...)
+    R_PRINTF_FORMAT(3,4);
+
+int Rasprintf_malloc(char **str, const char *fmt, ...)
+    R_PRINTF_FORMAT(2,3);
 
 SEXP fixup_NaRm(SEXP args); /* summary.c */
 void invalidate_cached_recodings(void);  /* from sysutils.c */
 void resetICUcollator(Rboolean disable); /* from util.c */
-void dt_invalidate_locale(); /* from Rstrptime.h */
+void dt_invalidate_locale(void); /* from Rstrptime.h */
 extern int R_OutputCon; /* from connections.c */
 extern int R_InitReadItemDepth, R_ReadItemDepth; /* from serialize.c */
 void get_current_mem(size_t *,size_t *,size_t *); /* from memory.c */
@@ -2175,6 +2311,8 @@ void R_CleanTempDir(void);
 #ifdef Win32
 void R_fixslash(char *s);
 void R_fixbackslash(char *s);
+void R_wfixbackslash(wchar_t *s);
+void R_wfixslash(wchar_t *s);
 wchar_t *filenameToWchar(const SEXP fn, const Rboolean expand);
 #endif
 
@@ -2189,6 +2327,10 @@ double R_atof(const char *str);
 /* unix/sys-std.c, main/options.c */
 void set_rl_word_breaks(const char *str);
 
+/* unix/sys-unix.c, main/connections.c */
+FILE *R_popen_pg(const char *cmd, const char *type);
+int R_pclose_pg(FILE *fp);
+
 /* From localecharset.c */
 extern const char *locale2charset(const char *);
 
@@ -2205,7 +2347,7 @@ extern const char *locale2charset(const char *);
 #  endif
 #  define gettext_noop(String) String
 #  define N_(String) gettext_noop (String)
-#  else /* not NLS */
+# else /* not NLS */
 #  define _(String) (String)
 #  define N_(String) String
 #  define ngettext(String, StringP, N) (N > 1 ? StringP: String)
@@ -2266,6 +2408,9 @@ extern void *alloca(size_t);
 // for reproducibility for now: use exp10 or pown later if accurate enough.
 #define Rexp10(x) pow(10.0, x)
 
+// this produces an initialized structure as a _compount literal_
+#define SEXP_TO_STACKVAL(x) ((R_bcstack_t) { .tag = 0, .u.sxpval = (x) })
+
 #endif /* DEFN_H_ */
 /*
  *- Local Variables:
