@@ -1,7 +1,7 @@
 #  File src/library/base/R/library.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2022 The R Core Team
+#  Copyright (C) 1995-2024 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -348,18 +348,16 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
             else canMaskEnv <- NULL
 
             if (attach.required)
-                .getRequiredPackages2(pkgInfo, quietly = quietly)
-
+                .getRequiredPackages2(pkgInfo, quietly = quietly,
+                                      lib.loc = c(lib.loc, .libPaths()))
             cr <- conflictRules(package)
             if (missing(mask.ok))
                 mask.ok <- cr$mask.ok
             if (missing(exclude))
                 exclude <- cr$exclude
 
-            ## If the namespace mechanism is available and the package
-            ## has a namespace, then the namespace loading mechanism
-            ## takes over.
-            if (packageHasNamespace(package, which.lib.loc)) {
+            ## The namespace loading mechanism takes over.
+
 		if (isNamespaceLoaded(package)) {
                     ## Already loaded.  Does the version match?
                     newversion <- as.numeric_version(pkgInfo$DESCRIPTION["Version"])
@@ -416,9 +414,6 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
                     else
                         return(invisible(.packages()))
                 }
-            } else
-		stop(gettextf("package %s does not have a namespace and should be re-installed",
-			      sQuote(package)), domain = NA)
 	}
 	if (verbose && !newpackage)
             warning(gettextf("package %s already present in search()",
@@ -712,6 +707,12 @@ function(package, lib.loc = NULL, quietly = FALSE, warn.conflicts,
     invisible(value)
 }
 
+use <-
+function(package, include.only)
+    invisible(library(package, lib.loc = NULL, character.only = TRUE,
+                      logical.return = TRUE, include.only = include.only,
+                      attach.required = FALSE))
+
 .packages <-
 function(all.available = FALSE, lib.loc = NULL)
 {
@@ -983,6 +984,7 @@ function(pkgInfo, quietly = FALSE, lib.loc = NULL, useImports = FALSE)
     }
 }
 
+## called e.g. w/ R_LIBS_USER  in  ../../profile/Common.R
 .expand_R_libs_env_var <-
 function(x)
 {
@@ -991,12 +993,15 @@ function(x)
     s <- Sys.info()
 
     R_LIBS_USER_default <- function() {
-        home <- normalizePath("~")
+        home <- normalizePath("~", mustWork = FALSE)  # possibly /nonexistent
         ## FIXME: could re-use v from "above".
-        x.y <- paste0(R.version$major, ".",
-                      sub("[.].*", "", R.version$minor))
-        if(.Platform$OS.type == "windows")
+        x.y <- paste(R.version$major, sep=".",
+                     strsplit(R.version$minor, ".", fixed=TRUE)[[1L]][1L])
+        if(.Platform$OS.type == "windows" && s["machine"] == "x86-64")
             file.path(Sys.getenv("LOCALAPPDATA"), "R", "win-library", x.y)
+        else if (.Platform$OS.type == "windows") # including aarch64
+            file.path(Sys.getenv("LOCALAPPDATA"), "R",
+                      paste0(s["machine"],"-library"), x.y)
         else if(s["sysname"] == "Darwin")
             file.path(home, "Library", "R", s["machine"], x.y, "library")
         else
@@ -1027,3 +1032,4 @@ function(x)
 
     gsub("%%", "%", x, fixed = TRUE)
 }
+
