@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
+ *  Copyright (C) 1997--2023  The R Core Team
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2014  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  *
  * EXPORTS:
  *
@@ -24,6 +24,7 @@
  *  vectorIndex()     -- used for "[[" and "[[<-" with a vector arg
 
  *  mat2indsub()      -- for "mat[i]"     "    "            "
+ *  strmat2intmat()   -- for "mat[<ch>]"  "    "            "
 
  *  makeSubscript()   -- for "[" and "[<-" in ./subset.c and ./subassign.c,
  *			 and "[[<-" with a scalar in ./subassign.c
@@ -36,8 +37,51 @@
 
 #include <Defn.h>
 
+#include <R_ext/Itermacros.h>
+
+/* interval at which to check interrupts, a guess (~subsecond on current hw) */
+#define NINTERRUPT 10000000
+
 /* We might get a call with R_NilValue from subassignment code */
-#define ECALL(call, yy) if(call == R_NilValue) error(yy); else errorcall(call, yy);
+#define ECALL(call, yy)     if(call == R_NilValue) error(yy);    else errorcall(call, yy);
+#define ECALL3(call, yy, A) if(call == R_NilValue) error(yy, A); else errorcall(call, yy, A);
+#define ECALL4(call, yy, A, B) if(call == R_NilValue) error(yy, A, B); else errorcall(call, yy, A, B);
+
+NORET static void ECALL_OutOfBounds(SEXP x, int subscript,
+				    R_xlen_t index, SEXP call)
+{
+    if (call == R_NilValue)
+	call = R_CurrentExpression; /* default behaves like error() */
+    SEXP sindex = ScalarReal((double) index);
+    PROTECT(sindex);
+    SEXP cond = R_makeOutOfBoundsError(x, subscript, sindex, call, NULL);
+    PROTECT(cond);
+    R_signalErrorCondition(cond, call);
+    UNPROTECT(2); /* sindex, cond; not reached */
+}
+
+NORET static void ECALL_MissingSubs(SEXP call) // no x
+{
+    if (call == R_NilValue)
+	call = R_CurrentExpression;
+    SEXP cond = R_makeMissingSubscriptError1(call);
+    PROTECT(cond);
+    R_signalErrorCondition(cond, call);
+    UNPROTECT(1); /* cond; not reached */
+}
+
+NORET static void ECALL_OutOfBoundsCHAR(SEXP x, int subscript,
+					SEXP sindex, SEXP call)
+{
+    if (call == R_NilValue)
+	call = R_CurrentExpression; /* default behaves like error() */
+    sindex = ScalarString(sindex);
+    PROTECT(sindex);
+    SEXP cond = R_makeOutOfBoundsError(x, subscript, sindex, call, NULL);
+    PROTECT(cond);
+    R_signalErrorCondition(cond, call);
+    UNPROTECT(2); /* sindex, cond; not reached */
+}
 
 /* This allows for the unusual case where x is of length 2,
    and x[[-m]] selects one element for m = 1, 2.
@@ -50,29 +94,29 @@ static R_INLINE int integerOneIndex(int i, R_xlen_t len, SEXP call)
     if (i > 0) /* a regular 1-based index from R */
 	indx = i - 1;
     else if (i == 0 || len < 2) {
-	ECALL(call, _("attempt to select less than one element"));
+	ECALL3(call, _("attempt to select less than one element in %s"), "integerOneIndex");
     } else if (len == 2 && i > -3)
 	indx = 2 + i;
     else {
-	ECALL(call, _("attempt to select more than one element"));
+	ECALL3(call, _("attempt to select more than one element in %s"), "integerOneIndex");
     }
     return indx;
 }
 
 /* Utility used (only in) do_subassign2_dflt(), i.e. "[[<-" in ./subassign.c : */
 R_xlen_t attribute_hidden
-OneIndex(SEXP x, SEXP s, R_xlen_t len, int partial, SEXP *newname, 
+OneIndex(SEXP x, SEXP s, R_xlen_t nx, int partial, SEXP *newname,
 	 int pos, SEXP call)
 {
     SEXP names;
-    R_xlen_t i, indx, nx;
+    R_xlen_t i, indx;
     const void *vmax;
 
     if (pos < 0 && length(s) > 1) {
-	ECALL(call, _("attempt to select more than one element"));
+	ECALL3(call, _("attempt to select more than one element in %s"), "OneIndex");
     }
     if (pos < 0 && length(s) < 1) {
-	ECALL(call, _("attempt to select less than one element"));
+	ECALL3(call, _("attempt to select less than one element in %s"), "OneIndex");
     }
 
     if(pos < 0) pos = 0;
@@ -82,16 +126,29 @@ OneIndex(SEXP x, SEXP s, R_xlen_t len, int partial, SEXP *newname,
     switch(TYPEOF(s)) {
     case LGLSXP:
     case INTSXP:
-	indx = integerOneIndex(INTEGER(s)[pos], len, call);
+	indx = integerOneIndex(INTEGER_ELT(s, pos), nx, call);
 	break;
     case REALSXP:
-	indx = integerOneIndex((int)REAL(s)[pos], len, call);
+    {
+	double dblind = REAL_ELT(s, pos);
+	if(!ISNAN(dblind)) {
+	    /* see comment above integerOneIndex */
+	    if (dblind >= 1) indx = (R_xlen_t)(dblind - 1);
+	    else if (dblind > -1 || nx < 2) {
+		ECALL3(call, _("attempt to select less than one element in %s"), "OneIndex <real>");
+	    } else if (nx == 2 && dblind > -3)
+		indx = 2 + (R_xlen_t) dblind;
+	    else {
+		ECALL3(call, _("attempt to select more than one element in %s"), "OneIndex <real>");
+	    }
+	}
 	break;
+    }
     case STRSXP:
 	vmax = vmaxget();
-	nx = xlength(x);
 	names = getAttrib(x, R_NamesSymbol);
 	if (names != R_NilValue) {
+	    PROTECT(names);
 	    /* Try for exact match */
 	    for (i = 0; i < nx; i++) {
 		const char *tmp = translateChar(STRING_ELT(names, i));
@@ -101,7 +158,7 @@ OneIndex(SEXP x, SEXP s, R_xlen_t len, int partial, SEXP *newname,
 		    break;
 		}
 	    }
-	    /* Try for partial match */
+	    // Try for partial match -- not ever used in current R (partial is 0)
 	    if (partial && indx < 0) {
 		size_t l = strlen(translateChar(STRING_ELT(s, pos)));
 		for(i = 0; i < nx; i++) {
@@ -115,6 +172,7 @@ OneIndex(SEXP x, SEXP s, R_xlen_t len, int partial, SEXP *newname,
 		    }
 		}
 	    }
+	    UNPROTECT(1); /* names */
 	}
 	if (indx == -1)
 	    indx = nx;
@@ -123,27 +181,24 @@ OneIndex(SEXP x, SEXP s, R_xlen_t len, int partial, SEXP *newname,
 	break;
     case SYMSXP:
 	vmax = vmaxget();
-	nx = xlength(x);
 	names = getAttrib(x, R_NamesSymbol);
 	if (names != R_NilValue) {
+	    PROTECT(names);
 	    for (i = 0; i < nx; i++)
 		if (streql(translateChar(STRING_ELT(names, i)),
-			   translateChar(PRINTNAME(s)))) {
+			   CHAR(PRINTNAME(s)))) {
 		    indx = i;
 		    break;
 		}
+	    UNPROTECT(1); /* names */
 	}
 	if (indx == -1)
 	    indx = nx;
-	*newname = STRING_ELT(s, pos);
+	*newname = PRINTNAME(s);
 	vmaxset(vmax);
 	break;
     default:
-	if (call == R_NilValue)
-	    error(_("invalid subscript type '%s'"), type2char(TYPEOF(s)));
-	else
-	    errorcall(call, _("invalid subscript type '%s'"),
-		      type2char(TYPEOF(s)));
+	ECALL3(call, _("invalid subscript type '%s'"), R_typeToChar(s));
     }
     return indx;
 }
@@ -163,49 +218,52 @@ get1index(SEXP s, SEXP names, R_xlen_t len, int pok, int pos, SEXP call)
    pok : is "partial ok" ?
 	 if pok is -1, warn if partial matching occurs, but allow.
 */
-    int  warn_pok = 0;
     const char *ss, *cur_name;
-    R_xlen_t indx;
     const void *vmax;
 
-    if (pok == -1) {
+    int warn_pok = (pok == -1);
+    if (warn_pok)
 	pok = 1;
-	warn_pok = 1;
-    }
 
     if (pos < 0 && length(s) != 1) {
 	if (length(s) > 1) {
-	    ECALL(call, _("attempt to select more than one element"));
+	    ECALL3(call, _("attempt to select more than one element in %s"), "get1index");
 	} else {
-	    ECALL(call, _("attempt to select less than one element"));
-	}
-    } else
-	if(pos >= length(s)) {
-	    ECALL(call, _("internal error in use of recursive indexing"));
+	    ECALL3(call, _("attempt to select less than one element in %s"), "get1index");
 	}
+    } else if(pos >= length(s)) {
+	ECALL(call, _("internal error in use of recursive indexing"));
+    }
     if(pos < 0) pos = 0;
-    indx = -1;
+    R_xlen_t indx = -1;
     switch (TYPEOF(s)) {
     case LGLSXP:
     case INTSXP:
     {
-	int i = INTEGER(s)[pos];
+	int i = INTEGER_ELT(s, pos);
 	if (i != NA_INTEGER)
 	    indx = integerOneIndex(i, len, call);
 	break;
     }
     case REALSXP:
     {
-	double dblind = REAL(s)[pos];
+	double dblind = REAL_ELT(s, pos);
 	if(!ISNAN(dblind)) {
 	    /* see comment above integerOneIndex */
-	    if (dblind > 0) indx = (R_xlen_t)(dblind - 1);
-	    else if (dblind == 0 || len < 2) {
-		ECALL(call, _("attempt to select less than one element"));
-	    } else if (len == 2 && dblind > -3)
-		indx = (R_xlen_t)(2 + dblind);
+	    if (dblind >= 1) {
+		if(R_FINITE(dblind)) indx = (R_xlen_t)(dblind - 1);
+	    } else if (dblind > -1 || len < 2) {
+		ECALL3(call,
+		       (dblind <= -1) ? _("invalid negative subscript in %s")
+		       : _("attempt to select less than one element in %s"),
+		       "get1index <real>");
+	    } else if (len == 2 && dblind > -3) // dblind = -2 or -1 {why exception ?}
+		indx = 2 + (R_xlen_t) dblind;
 	    else {
-		ECALL(call, _("attempt to select more than one element"));
+		ECALL3(call,
+		       (dblind <= -1) ? _("invalid negative subscript in %s")
+		       : _("attempt to select more than one element in %s"),
+		       "get1index <real>");
 	    }
 	}
 	break;
@@ -260,6 +318,9 @@ get1index(SEXP s, SEXP names, R_xlen_t len, int pok, int pos, SEXP call)
 	vmaxset(vmax);
 	break;
     case SYMSXP:
+	if (s == R_MissingArg) {
+	    ECALL_MissingSubs(call);
+	}
 	vmax = vmaxget();
 	for (R_xlen_t i = 0; i < xlength(names); i++)
 	    if (STRING_ELT(names, i) != NA_STRING &&
@@ -269,24 +330,21 @@ get1index(SEXP s, SEXP names, R_xlen_t len, int pok, int pos, SEXP call)
 		vmaxset(vmax);
 		break;
 	    }
+	break;
     default:
-	if (call == R_NilValue)
-	    error(_("invalid subscript type '%s'"), type2char(TYPEOF(s)));
-	else
-	    errorcall(call, _("invalid subscript type '%s'"),
-		      type2char(TYPEOF(s)));
+	ECALL3(call, _("invalid subscript type '%s'"), R_typeToChar(s));
     }
     return indx;
 }
 
 /* This is used for [[ and [[<- with a vector of indices of length > 1 .
-   x is a list or pairlist, and it is indexed recusively from 
+   x is a list or pairlist, and it is indexed recusively from
    level start to level stop-1.  ( 0...len-1 or 0..len-2 then len-1).
    For [[<- it needs to duplicate if substructure might be shared.
  */
-SEXP attribute_hidden
+attribute_hidden SEXP
 vectorIndex(SEXP x, SEXP thesub, int start, int stop, int pok, SEXP call,
-	    Rboolean dup) 
+	    Rboolean dup)
 {
     int i;
     R_xlen_t offset;
@@ -301,10 +359,13 @@ vectorIndex(SEXP x, SEXP thesub, int start, int stop, int pok, SEXP call,
 	    if (i)
 		errorcall(call, _("recursive indexing failed at level %d\n"), i+1);
 	    else
-		errorcall(call, _("attempt to select more than one element"));
+		errorcall(call, _("attempt to select more than one element in %s"), "vectorIndex");
 	}
-	offset = get1index(thesub, getAttrib(x, R_NamesSymbol),
-		           xlength(x), pok, i, call);
+	PROTECT(x);
+	SEXP names = PROTECT(getAttrib(x, R_NamesSymbol));
+	offset = get1index(thesub, names,
+			   xlength(x), pok, i, call);
+	UNPROTECT(2); /* x, names */
 	if(offset < 0 || offset >= xlength(x))
 	    errorcall(call, _("no such index at level %d\n"), i+1);
 	if(isPairList(x)) {
@@ -313,23 +374,25 @@ vectorIndex(SEXP x, SEXP thesub, int start, int stop, int pok, SEXP call,
 		error("invalid subscript for pairlist");
 #endif
 	    cx = nthcdr(x, (int) offset);
-	    if (NAMED(x) > NAMED(CAR(cx)))
-		SET_NAMED(CAR(x), NAMED(x));
+	    RAISE_NAMED(CAR(x), NAMED(x));
 	    x = CAR(cx);
 	    if (dup && MAYBE_SHARED(x)) {
+		PROTECT(cx);
 		x = shallow_duplicate(x);
 		SETCAR(cx, x);
+		UNPROTECT(1); /* cx */
 	    }
 	} else {
 	    cx = x;
 	    x = VECTOR_ELT(x, offset);
-	    if (NAMED(cx) > NAMED(x))
-		SET_NAMED(x, NAMED(cx));
+	    RAISE_NAMED(x, NAMED(cx));
 	    if (dup && MAYBE_SHARED(x)) {
+		PROTECT(cx);
 		x = shallow_duplicate(x);
 		SET_VECTOR_ELT(cx, offset, x);
+		UNPROTECT(1); /* cx */
 	    }
-    	}
+	}
     }
     return x;
 }
@@ -344,22 +407,29 @@ vectorIndex(SEXP x, SEXP thesub, int start, int stop, int pok, SEXP call,
    A zero/NA anywhere in a row will cause a zero/NA in the same
    position in the result.
 */
+/*
+  Called from `[`   VectorSubset()    ./subset.c	and
+              `[<-` VectorSubassign() ./subassign.c
 
-
-SEXP attribute_hidden mat2indsub(SEXP dims, SEXP s, SEXP call)
+  only if(isMatrix(s) && isArray(x) && ncols(s) == "length(dim(x))"),
+  where dims = getAttrib(x, R_DimSymbol);
+*/
+attribute_hidden SEXP mat2indsub(SEXP dims, SEXP s, SEXP call, SEXP x)
 {
     int nrs = nrows(s);
     R_xlen_t NR = nrs;
     SEXP rvec;
+    int ndim = LENGTH(dims);
+    const int *pdims = INTEGER_RO(dims);
 
-    if (ncols(s) != LENGTH(dims)) {
+    if (ncols(s) != ndim) {
 	ECALL(call, _("incorrect number of columns in matrix subscript"));
     }
 
 #ifdef LONG_VECTOR_SUPPORT
     /* Check if it is a long vector we need to index */
     R_xlen_t len = 1;
-    for (int j = 0; j < LENGTH(dims); j++)  len *= INTEGER(dims)[j];
+    for (int j = 0; j < ndim; j++)  len *= pdims[j];
 
     if(len > R_SHORT_LEN_MAX) {
 	PROTECT(rvec = allocVector(REALSXP, nrs));
@@ -368,36 +438,39 @@ SEXP attribute_hidden mat2indsub(SEXP dims, SEXP s, SEXP call)
 	if (TYPEOF(s) == REALSXP) {
 	    for (int i = 0; i < nrs; i++) {
 		R_xlen_t tdim = 1;
-		for (int j = 0; j < LENGTH(dims); j++) {
-		    double k = REAL(s)[i + j * NR];
+		const double *ps = REAL_RO(s);
+		for (int j = 0; j < ndim; j++) {
+		    double k = ps[i + j * NR];
 		    if(ISNAN(k)) {rv[i] = NA_REAL; break;}
+		    k = trunc(k); // = "as.integer(k)" for large k
 		    if(k < 0) {
 			ECALL(call, _("negative values are not allowed in a matrix subscript"));
 		    }
 		    if(k == 0.) {rv[i] = 0.; break;}
-		    if (k > INTEGER(dims)[j]) {
-			ECALL(call, _("subscript out of bounds"));
+		    if (k > pdims[j]) {
+			ECALL_OutOfBounds(x, j, (R_xlen_t)k, call);
 		    }
 		    rv[i] += (k - 1.) * tdim;
-		    tdim *= INTEGER(dims)[j];
+		    tdim *= pdims[j];
 		}
 	    }
 	} else {
 	    s = coerceVector(s, INTSXP);
+	    const int *ps = INTEGER_RO(s);
 	    for (int i = 0; i < nrs; i++) {
 		R_xlen_t tdim = 1;
-		for (int j = 0; j < LENGTH(dims); j++) {
-		    int k = INTEGER(s)[i + j * NR];
+		for (int j = 0; j < ndim; j++) {
+		    int k = ps[i + j * NR];
 		    if(k == NA_INTEGER) {rv[i] = NA_REAL; break;}
 		    if(k < 0) {
 			ECALL(call, _("negative values are not allowed in a matrix subscript"));
 		    }
 		    if(k == 0) {rv[i] = 0.; break;}
-		    if (k > INTEGER(dims)[j]) {
-			ECALL(call, _("subscript out of bounds"));
+		    if (k > pdims[j]) {
+			ECALL_OutOfBounds(x, j, k, call);
 		    }
 		    rv[i] += (double) ((k - 1) * tdim);
-		    tdim *= INTEGER(dims)[j];
+		    tdim *= pdims[j];
 		}
 	    }
 	}
@@ -408,20 +481,21 @@ SEXP attribute_hidden mat2indsub(SEXP dims, SEXP s, SEXP call)
 	int *iv = INTEGER(rvec);
 	for (int i = 0; i < nrs; i++) iv[i] = 1; // 1-based.
 	s = coerceVector(s, INTSXP);
+	int *ps = INTEGER(s);
 	for (int i = 0; i < nrs; i++) {
 	    int tdim = 1;
-	    for (int j = 0; j < LENGTH(dims); j++) {
-		int k = INTEGER(s)[i + j * NR];
+	    for (int j = 0; j < ndim; j++) {
+		int k = ps[i + j * NR];
 		if(k == NA_INTEGER) {iv[i] = NA_INTEGER; break;}
 		if(k < 0) {
 		    ECALL(call, _("negative values are not allowed in a matrix subscript"));
 		}
 		if(k == 0) {iv[i] = 0; break;}
-		if (k > INTEGER(dims)[j]) {
-		    ECALL(call, _("subscript out of bounds"));
+		if (k > pdims[j]) {
+		    ECALL_OutOfBounds(x, j, k, call);
 		}
 		iv[i] += (k - 1) * tdim;
-		tdim *= INTEGER(dims)[j];
+		tdim *= pdims[j];
 	    }
 	}
     }
@@ -436,31 +510,57 @@ array and i is a character matrix with n columns, this code converts i
 to an integer matrix by matching against the dimnames of x. NA values
 in any row of i propagate to the result.  Unmatched entries result in
 a subscript out of bounds error.  */
+/*
+  Called from `[`   VectorSubset()    ./subset.c	and
+              `[<-` VectorSubassign() ./subassign.c
 
-SEXP attribute_hidden strmat2intmat(SEXP s, SEXP dnamelist, SEXP call)
+  only if(isMatrix(s) && isArray(x) && ncols(s) == "length(dim(x))"),
+  where dnamelist = PROTECT(GetArrayDimnames(x))
+ */
+attribute_hidden SEXP strmat2intmat(SEXP s, SEXP dnamelist, SEXP call, SEXP x)
 {
     /* XXX: assumes all args are protected */
-    int nr = nrows(s), i, j, v;
-    R_xlen_t idx, NR = nr;
-    SEXP dnames, snames, si, sicol, s_elt;
-    PROTECT(snames = allocVector(STRSXP, nr));
-    PROTECT(si = allocVector(INTSXP, xlength(s)));
-    dimgets(si, getAttrib(s, R_DimSymbol));
-    for (i = 0; i < length(dnamelist); i++) {
-        dnames = VECTOR_ELT(dnamelist, i);
-        for (j = 0; j < nr; j++)
-            SET_STRING_ELT(snames, j, STRING_ELT(s, j + (i * NR)));
-        PROTECT(sicol = match(dnames, snames, 0));
-        for (j = 0; j < nr; j++) {
-            v = INTEGER(sicol)[j];
-            idx = j + (i * NR);
-            s_elt = STRING_ELT(s, idx);
-            if (s_elt == NA_STRING) v = NA_INTEGER;
-            if (!CHAR(s_elt)[0]) v = 0; /* disallow "" match */
-            if (v == 0) errorcall(call, _("subscript out of bounds"));
-            INTEGER(si)[idx] = v;
-        }
-        UNPROTECT(1);
+    SEXP dim = getAttrib(s, R_DimSymbol);
+    int nr = INTEGER(dim)[0], // = nrow(s)
+	nc = INTEGER(dim)[1]; // = ncol(s)
+    if(isNull(dnamelist)) { // && nr > 0)
+#if 1
+	ECALL(call, _("no 'dimnames' attribute for array")); // as in  arraySubscript()
+#else
+	ECALL4(call,
+          _("%d x %d character matrix subscript illegal for NULL dimnames"),
+	       nr, nc);
+#endif
+    }
+#if 0 /* unneeded as only called  if (length(dnamelist) == nc) : */
+    if(length(dnamelist) != nc) {
+	ECALL4(call,
+          _("subscripting by character matrix of %d columns does not match length(dimnames(x)) == %d"),
+	       nc, length(dnamelist));
+    }
+#endif
+    SEXP snames = PROTECT(allocVector(STRSXP, nr));
+    // result si := an integer matrix of same dimension as 's'
+    SEXP si = PROTECT(allocVector(INTSXP, xlength(s)));
+    dimgets(si, dim);
+    int *psi = INTEGER(si);
+    memset(psi, 0, XLENGTH(si) * sizeof(int));
+    for (int i = 0; i < nc; i++) {
+	R_xlen_t iNR = i * (R_xlen_t) nr;
+	for (int j = 0; j < nr; j++)
+	    SET_STRING_ELT(snames, j, STRING_ELT(s, j + iNR));
+	SEXP dnames = VECTOR_ELT(dnamelist, i);
+	SEXP sicol = PROTECT(match(dnames, snames, 0));
+	for (int j = 0; j < nr; j++) {
+	    int v = INTEGER_ELT(sicol, j);
+	    R_xlen_t idx = j + iNR;
+	    SEXP s_elt = STRING_ELT(s, idx);
+	    if (s_elt == NA_STRING) v = NA_INTEGER;
+	    else if (!CHAR(s_elt)[0]) v = 0; /* disallow "" match */
+	    if (v == 0) ECALL_OutOfBoundsCHAR(x, i, s_elt, call);
+	    psi[idx] = v;
+	}
+	UNPROTECT(1);
     }
     UNPROTECT(2);
     return si;
@@ -471,24 +571,26 @@ static SEXP nullSubscript(R_xlen_t n)
     SEXP indx;
 #ifdef LONG_VECTOR_SUPPORT
     if (n > R_SHORT_LEN_MAX) {
- 	indx = allocVector(REALSXP, n);
+	indx = allocVector(REALSXP, n);
+	double *pindx = REAL(indx);
 	for (R_xlen_t i = 0; i < n; i++)
-	    REAL(indx)[i] = (double)(i + 1);
-    } else 
+	    pindx[i] = (double)(i + 1);
+    } else
 #endif
     {
 	indx = allocVector(INTSXP, n);
+	int *pindx = INTEGER(indx);
 	for (int i = 0; i < n; i++)
-	    INTEGER(indx)[i] = i + 1;
+	    pindx[i] = i + 1;
     }
     return indx;
 }
 
 
-static SEXP 
+static SEXP
 logicalSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, R_xlen_t *stretch, SEXP call)
 {
-    R_xlen_t count, i, nmax;
+    R_xlen_t count, i, nmax, i1, i2;
     int canstretch;
     SEXP indx;
     canstretch = *stretch > 0;
@@ -498,35 +600,112 @@ logicalSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, R_xlen_t *stretch, SEXP call)
     nmax = (ns > nx) ? ns : nx;
     *stretch = (ns > nx) ? ns : 0;
     if (ns == 0) return(allocVector(INTSXP, 0));
+    const int *ps = LOGICAL_RO(s);    /* Calling LOCICAL_RO here may force a
+					 large allocation, but no larger than
+					 the one made by R_alloc below. This
+					 could use rewriting to better handle
+					 a sparse logical index. */
 #ifdef LONG_VECTOR_SUPPORT
     if (nmax > R_SHORT_LEN_MAX) {
+	if (ns == nmax) { /* no recycling - use fast single-index code */
+	    const void *vmax = vmaxget();
+	    double *buf = (double *) R_alloc(nmax, sizeof(double));
+	    count = 0;
+	    R_ITERATE_CHECK(NINTERRUPT, nmax, i,
+		if (ps[i]) {
+		    if (ps[i] == NA_LOGICAL)
+			buf[count++] = NA_REAL;
+		    else
+			buf[count++] = (double)(i + 1);
+		});
+	    PROTECT(indx = allocVector(REALSXP, count));
+	    memcpy(REAL(indx), buf, sizeof(double) * count);
+	    vmaxset(vmax);
+	    UNPROTECT(1);
+	    return indx;
+	}
 	count = 0;
-	for (R_xlen_t i = 0; i < nmax; i++)
-	    if (LOGICAL(s)[i%ns]) count++;
-	indx = allocVector(REALSXP, count);
-	count = 0;
-	for (i = 0; i < nmax; i++)
-	    if (LOGICAL(s)[i%ns]) {
-		if (LOGICAL(s)[i%ns] == NA_LOGICAL)
-		    REAL(indx)[count++] = NA_REAL;
-		else
-		    REAL(indx)[count++] = (double)(i + 1);
+	/* we only need to scan s once even if we recycle,
+	   just remember the total count as well as
+	   the count for the last incomplete chunk (if any) */
+	i1 = (ns < nmax) ? (nmax % ns) : 0;
+	if (i1 > 0) { /* last recycling chunk is incomplete -
+			 we have to get the truncated count as well */
+	    R_xlen_t rem = 0;
+	    for (i = 0; i < ns; i++) {
+		if (i == i1) rem = count;
+		if (ps[i]) count++;
 	    }
+	    count = count * (nmax / ns) + rem;
+	} else { /* nested recycling, total is sufficient */
+	    for (i = 0; i < ns; i++)
+		if (ps[i]) count++;
+	    count *= nmax / ns;
+	}
+	PROTECT(indx = allocVector(REALSXP, count));
+	double *pindx = REAL(indx);
+	count = 0;
+	MOD_ITERATE_CHECK(NINTERRUPT, nmax, ns, nmax, i, i1, i2,
+	    if (ps[i1]) {
+		if (ps[i1] == NA_LOGICAL)
+		    pindx[count++] = NA_REAL;
+		else
+		    pindx[count++] = (double)(i + 1);
+	    });
+
+	UNPROTECT(1);
 	return indx;
     }
 #endif
+// else --- the same code for  non-long vectors --------------------------
+    if (ns == nmax) {  /* no recycling - use fast single-index code */
+	const void *vmax = vmaxget();
+	int *buf = (int *) R_alloc(nmax, sizeof(int));
+	count = 0;
+	R_ITERATE_CHECK(NINTERRUPT, nmax, i,
+	    if (ps[i]) {
+		if (ps[i] == NA_LOGICAL)
+		    buf[count++] = NA_INTEGER;
+		else
+		    buf[count++] = (int)(i + 1);
+	    });
+	PROTECT(indx = allocVector(INTSXP, count));
+	memcpy(INTEGER(indx), buf, sizeof(int) * count);
+	vmaxset(vmax);
+	UNPROTECT(1);
+	return indx;
+    }
+
     count = 0;
-    for (i = 0; i < nmax; i++)
-	if (LOGICAL(s)[i%ns]) count++;
-    indx = allocVector(INTSXP, count);
-    count = 0;
-    for (i = 0; i < nmax; i++)
-	if (LOGICAL(s)[i%ns]) {
-	    if (LOGICAL(s)[i%ns] == NA_LOGICAL)
-		INTEGER(indx)[count++] = NA_INTEGER;
-	    else
-		INTEGER(indx)[count++] = (int)(i + 1);
+    /* we only need to scan s once even if we recycle,
+       just remember the total count as well as
+       the count for the last incomplete chunk (if any) */
+    i1 = (ns < nmax) ? (nmax % ns) : 0;
+    if (i1 > 0) { /* last recycling chunk is incomplete -
+		     we have to get the truncated count as well */
+	R_xlen_t rem = 0;
+	for (i = 0; i < ns; i++) {
+	    if (i == i1) rem = count;
+	    if (ps[i]) count++;
 	}
+	count = count * (nmax / ns) + rem;
+    } else { /* nested recycling, total is sufficient */
+	for (i = 0; i < ns; i++)
+	    if (ps[i]) count++;
+	count *= nmax / ns;
+    }
+    PROTECT(indx = allocVector(INTSXP, count));
+    int *pindx = INTEGER(indx);
+    count = 0;
+    MOD_ITERATE_CHECK(NINTERRUPT, nmax, ns, nmax, i, i1, i2,
+	if (ps[i1]) {
+	    if (ps[i1] == NA_LOGICAL)
+		pindx[count++] = NA_INTEGER;
+	    else
+		pindx[count++] = (int)(i + 1);
+	});
+
+    UNPROTECT(1);
     return indx;
 }
 
@@ -536,12 +715,14 @@ static SEXP negativeSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, SEXP call)
     R_xlen_t stretch = 0;
     R_xlen_t i;
     PROTECT(indx = allocVector(LGLSXP, nx));
+    int *pindx = LOGICAL(indx);
     for (i = 0; i < nx; i++)
-	LOGICAL(indx)[i] = 1;
+	pindx[i] = 1;
+    const int *ps = INTEGER_RO(s);
     for (i = 0; i < ns; i++) {
-	int ix = INTEGER(s)[i];
+	int ix = ps[i];
 	if (ix != 0 && ix != NA_INTEGER && -ix <= nx)
-	    LOGICAL(indx)[-ix - 1] = 0;
+	    pindx[-ix - 1] = 0;
     }
     s = logicalSubscript(indx, nx, nx, &stretch, call);
     UNPROTECT(1);
@@ -552,43 +733,49 @@ static SEXP positiveSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx)
 {
     SEXP indx;
     R_xlen_t i, zct = 0;
-    for (i = 0; i < ns; i++) if (INTEGER(s)[i] == 0) zct++;
+    const int *ps = INTEGER_RO(s);
+    for (i = 0; i < ns; i++) if (ps[i] == 0) zct++;
     if (zct) {
 	indx = allocVector(INTSXP, (ns - zct));
+	int *pindx = INTEGER(indx);
 	for (i = 0, zct = 0; i < ns; i++)
-	    if (INTEGER(s)[i] != 0)
-		INTEGER(indx)[zct++] = INTEGER(s)[i];
+	    if (ps[i] != 0)
+		pindx[zct++] = ps[i];
 	return indx;
 
     } else return s;
 }
 
-static SEXP 
-integerSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, R_xlen_t *stretch, SEXP call)
+static SEXP
+integerSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, R_xlen_t *stretch,
+		 SEXP call, SEXP x)
 {
     R_xlen_t i;
-    int ii, min, max, canstretch;
+    int ii, neg, max, canstretch;
     Rboolean isna = FALSE;
     canstretch = *stretch > 0;
     *stretch = 0;
-    min = 0;
+    neg = FALSE;
     max = 0;
+    const int *ps = INTEGER_RO(s);
     for (i = 0; i < ns; i++) {
-	ii = INTEGER(s)[i];
-	if (ii != NA_INTEGER) {
-	    if (ii < min)
-		min = ii;
-	    if (ii > max)
-		max = ii;
-	} else isna = TRUE;
+	ii = ps[i];
+	if (ii < 0) {
+	    if (ii == NA_INTEGER)
+		isna = TRUE;
+	    else
+		neg = TRUE;
+	}
+	else if (ii > max)
+	    max = ii;
     }
     if (max > nx) {
 	if(canstretch) *stretch = max;
 	else {
-	    ECALL(call, _("subscript out of bounds"));
+	    ECALL_OutOfBounds(x, -1, max, call);
 	}
     }
-    if (min < 0) {
+    if (neg) {
 	if (max == 0 && !isna) return negativeSubscript(s, ns, nx, call);
 	else {
 	    ECALL(call, _("only 0's may be mixed with negative subscripts"));
@@ -598,25 +785,23 @@ integerSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, R_xlen_t *stretch, SEXP call)
     return R_NilValue;
 }
 
-static SEXP 
-realSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, R_xlen_t *stretch, SEXP call)
+static SEXP
+realSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, R_xlen_t *stretch,
+	      SEXP call, SEXP x)
 {
-    R_xlen_t i;
-    int canstretch;
-    double ii, min, max;
-    Rboolean isna = FALSE;
-    canstretch = *stretch > 0;
+    int canstretch = *stretch > 0;
     *stretch = 0;
-    min = 0;
-    max = 0;
-    for (i = 0; i < ns; i++) {
-	ii = REAL(s)[i];
+    double min = 0, max = 0;
+    const double *ps = REAL_RO(s);
+    Rboolean isna = FALSE;
+    for (R_xlen_t i = 0; i < ns; i++) {
+	double ii = ps[i];
 	if (R_FINITE(ii)) {
 	    if (ii < min) min = ii;
 	    if (ii > max) max = ii;
 	} else isna = TRUE;
     }
-    if (max > nx) {
+    if (max >= nx+1.) {
 #ifndef LONG_VECTOR_SUPPORT
 	if (max > INT_MAX) {
 	    ECALL(call, _("subscript too large for 32-bit R"));
@@ -624,22 +809,20 @@ realSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, R_xlen_t *stretch, SEXP call)
 #endif
 	if(canstretch) *stretch = (R_xlen_t) max;
 	else {
-	    ECALL(call, _("subscript out of bounds"));
+	    ECALL_OutOfBounds(x, -1, (R_xlen_t) max, call);
 	}
     }
-    if (min < 0) {
-	if (max == 0 && !isna) {
-	    SEXP indx;
-	    R_xlen_t stretch = 0;
-	    double dx;
-	    R_xlen_t i, ix;
-	    PROTECT(indx = allocVector(LGLSXP, nx));
-	    for (i = 0; i < nx; i++) LOGICAL(indx)[i] = 1;
+    if (min <= -1) {
+	if (max < 1 && !isna) {
+	    R_xlen_t i, stretch = 0;
+	    SEXP indx = PROTECT(allocVector(LGLSXP, nx));
+	    int *pindx = LOGICAL(indx);
+	    for (i = 0; i < nx; i++) pindx[i] = 1;
 	    for (i = 0; i < ns; i++) {
-		dx = REAL(s)[i];
-		if (R_FINITE(dx) && dx != 0  && -dx <= nx) {
-		    ix = (int)(-dx - 1);
-		    LOGICAL(indx)[ix] = 0;
+		double dx = ps[i];
+		if (R_FINITE(dx) && dx <= -1  && -dx < nx+1.) {
+		    R_xlen_t ix = (R_xlen_t)(-dx - 1);
+		    pindx[ix] = 0;
 		}
 	    }
 	    s = logicalSubscript(indx, nx, nx, &stretch, call);
@@ -656,7 +839,7 @@ realSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, R_xlen_t *stretch, SEXP call)
 	/* NB, indices will be truncated eventually,
 	   so need to do that to take '0' into account */
 	for (i = 0; i < ns; i++) {
-	    double ds = REAL(s)[i];
+	    double ds = ps[i];
 #ifdef OLDCODE_LONG_VECTOR
 	    if (!R_FINITE(ds)) {
 		if (ds > INT_MAX) int_ok = FALSE;
@@ -669,18 +852,20 @@ realSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, R_xlen_t *stretch, SEXP call)
 	}
 	if (int_ok) {
 	    indx = allocVector(INTSXP, cnt);
+	    int *pindx = INTEGER(indx);
 	    for (i = 0, cnt = 0; i < ns; i++) {
-		double ds = REAL(s)[i];
+		double ds = ps[i];
 		int ia;
 		if (!R_FINITE(ds)) ia = NA_INTEGER;
 		else ia = (int) ds;
-		if (ia != 0) INTEGER(indx)[cnt++] = ia;
+		if (ia != 0) pindx[cnt++] = ia;
 	    }
 	} else {
 	    indx = allocVector(REALSXP, cnt);
+	    double *pindx = REAL(indx);
 	    for (i = 0, cnt = 0; i < ns; i++) {
-		double ds = REAL(s)[i];
-		if (!R_FINITE(ds) || (R_xlen_t) ds != 0) REAL(indx)[cnt++] = ds;
+		double ds = ps[i];
+		if (!R_FINITE(ds) || (R_xlen_t) ds != 0) pindx[cnt++] = ds;
 	    }
 	}
 	return indx;
@@ -705,17 +890,18 @@ realSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, R_xlen_t *stretch, SEXP call)
 
 static SEXP
 stringSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, SEXP names,
-		R_xlen_t *stretch, SEXP call)
+		R_xlen_t *stretch, SEXP call, SEXP x, int dim)
 {
-    SEXP indx, indexnames;
+    SEXP indx, indexnames = R_NilValue;
     R_xlen_t i, j, nnames, extra, sub;
     int canstretch = *stretch > 0;
     /* product may overflow, so check factors as well. */
     Rboolean usehashing = ( ((ns > 1000 && nx) || (nx > 1000 && ns)) || (ns * nx > 15*nx + ns) );
+    int nprotect = 0;
 
     PROTECT(s);
     PROTECT(names);
-    PROTECT(indexnames = allocVector(VECSXP, ns));
+    nprotect += 2;
     nnames = nx;
     extra = nnames;
 
@@ -730,14 +916,17 @@ stringSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, SEXP names,
 	/* must be internal, so names contains a character vector */
 	/* NB: this does not behave in the same way with respect to ""
 	   and NA names: they will match */
-	PROTECT(indx = match(names, s, 0));
+	PROTECT(indx = match(names, s, 0)); /**** guaranteed to be fresh???*/
+	nprotect++;
 	/* second pass to correct this */
+	int *pindx = INTEGER(indx);
 	for (i = 0; i < ns; i++)
 	    if(STRING_ELT(s, i) == NA_STRING || !CHAR(STRING_ELT(s, i))[0])
-		INTEGER(indx)[i] = 0;
-	for (i = 0; i < ns; i++) SET_VECTOR_ELT(indexnames, i, R_NilValue);
+		pindx[i] = 0;
     } else {
 	PROTECT(indx = allocVector(INTSXP, ns));
+	nprotect++;
+	int *pindx = INTEGER(indx);
 	for (i = 0; i < ns; i++) {
 	    sub = 0;
 	    if (names != R_NilValue) {
@@ -745,35 +934,41 @@ stringSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, SEXP names,
 		    SEXP names_j = STRING_ELT(names, j);
 		    if (NonNullStringMatch(STRING_ELT(s, i), names_j)) {
 			sub = j + 1;
-			SET_VECTOR_ELT(indexnames, i, R_NilValue);
 			break;
 		    }
 		}
 	    }
-	    INTEGER(indx)[i] = (int) sub;
+	    pindx[i] = (int) sub;
 	}
     }
 
-
+    int *pindx = INTEGER(indx);
+    SEXP sindx = NULL;
     for (i = 0; i < ns; i++) {
-	sub = INTEGER(indx)[i];
+	sub = pindx[i];
 	if (sub == 0) {
-	    for (j = 0 ; j < i ; j++)
-		if (NonNullStringMatch(STRING_ELT(s, i), STRING_ELT(s, j))) {
-		    sub = INTEGER(indx)[j];
-		    SET_VECTOR_ELT(indexnames, i, STRING_ELT(s, j));
-		    break;
-		}
+	    if (sindx == NULL) {
+		sindx = PROTECT(match(s, s, 0));
+		indexnames = PROTECT(allocVector(VECSXP, ns));
+		nprotect += 2;
+		for (int z = 0; z < ns; z++)
+		    SET_VECTOR_ELT(indexnames, z, R_NilValue);
+	    }
+	    int j = INTEGER(sindx)[i] - 1;
+	    if(STRING_ELT(s, i) != NA_STRING && CHAR(STRING_ELT(s, i))[0]) {
+		sub = pindx[j];
+		SET_VECTOR_ELT(indexnames, i, STRING_ELT(s, j));
+	    }
 	}
 	if (sub == 0) {
 	    if (!canstretch) {
-		ECALL(call, _("subscript out of bounds"));
+		ECALL_OutOfBoundsCHAR(x, dim, STRING_ELT(s, i), call);
 	    }
 	    extra += 1;
 	    sub = extra;
 	    SET_VECTOR_ELT(indexnames, i, STRING_ELT(s, i));
 	}
-	INTEGER(indx)[i] = (int) sub;
+	pindx[i] = (int) sub;
     }
     /* We return the new names as the names attribute of the returned
        subscript vector. */
@@ -781,7 +976,7 @@ stringSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, SEXP names,
 	setAttrib(indx, R_UseNamesSymbol, indexnames);
     if (canstretch)
 	*stretch = extra;
-    UNPROTECT(4);
+    UNPROTECT(nprotect);
     return indx;
 }
 
@@ -800,7 +995,7 @@ int_arraySubscript(int dim, SEXP s, SEXP dims, SEXP x, SEXP call)
     R_xlen_t stretch = 0;
     SEXP dnames, tmp;
     ns = length(s);
-    nd = INTEGER(dims)[dim];
+    nd = INTEGER_ELT(dims, dim);
 
     switch (TYPEOF(s)) {
     case NILSXP:
@@ -808,11 +1003,11 @@ int_arraySubscript(int dim, SEXP s, SEXP dims, SEXP x, SEXP call)
     case LGLSXP:
 	return logicalSubscript(s, ns, nd, &stretch, call);
     case INTSXP:
-	return integerSubscript(s, ns, nd, &stretch, call);
+	return integerSubscript(s, ns, nd, &stretch, call, x);
     case REALSXP:
 	/* We don't yet allow subscripts > R_SHORT_LEN_MAX */
 	PROTECT(tmp = coerceVector(s, INTSXP));
-	tmp = integerSubscript(tmp, ns, nd, &stretch, call);
+	tmp = integerSubscript(tmp, ns, nd, &stretch, call, x);
 	UNPROTECT(1);
 	return tmp;
     case STRSXP:
@@ -821,16 +1016,12 @@ int_arraySubscript(int dim, SEXP s, SEXP dims, SEXP x, SEXP call)
 	    ECALL(call, _("no 'dimnames' attribute for array"));
 	}
 	dnames = VECTOR_ELT(dnames, dim);
-	return stringSubscript(s, ns, nd, dnames, &stretch, call);
+	return stringSubscript(s, ns, nd, dnames, &stretch, call, x, dim);
     case SYMSXP:
 	if (s == R_MissingArg)
 	    return nullSubscript(nd);
     default:
-	if (call == R_NilValue)
-	    error(_("invalid subscript type '%s'"), type2char(TYPEOF(s)));
-	else
-	    errorcall(call, _("invalid subscript type '%s'"),
-		      type2char(TYPEOF(s)));
+	ECALL3(call, _("invalid subscript type '%s'"), R_typeToChar(s));
     }
     return R_NilValue;
 }
@@ -854,7 +1045,7 @@ arraySubscript(int dim, SEXP s, SEXP dims, AttrGetter dng,
    otherwise, stretch returns the new required length for x
 */
 
-SEXP attribute_hidden
+attribute_hidden SEXP
 makeSubscript(SEXP x, SEXP s, R_xlen_t *stretch, SEXP call)
 {
     if (! (isVector(x) || isList(x) || isLanguage(x))) {
@@ -862,34 +1053,28 @@ makeSubscript(SEXP x, SEXP s, R_xlen_t *stretch, SEXP call)
     }
 
     R_xlen_t nx = xlength(x);
-    R_xlen_t ns = xlength(s);
 
     /* special case for simple indices -- does not duplicate */
-    if (ns == 1) {
-	if (TYPEOF(s) == INTSXP) {
-	    int i = INTEGER(s)[0];
-	    if (0 < i && i <= nx) {
-		*stretch = 0;
-		return s;
-	    }
+    if (IS_SCALAR(s, INTSXP)) {
+	int i = SCALAR_IVAL(s);
+	if (0 < i && i <= nx) {
+	    *stretch = 0;
+	    return s;
 	}
-	else if (TYPEOF(s) == REALSXP) {
-	    double di = REAL(s)[0];
-	    if (1 <= di && di <= nx) {
-		*stretch = 0;
-		/* We could only return a REALSXP if the value is too
-		   large for an INTSXP, but, as the calling code can
-		   handle REALSXP indices, returning the REALSXP
-		   avoids and allocation. */
-		return s;
-	    }
+    }
+    else if (IS_SCALAR(s, REALSXP)) {
+	double di = SCALAR_DVAL(s);
+	if (1 <= di && di <= nx) {
+	    *stretch = 0;
+	    /* We could only return a REALSXP if the value is too
+	       large for an INTSXP, but, as the calling code can
+	       handle REALSXP indices, returning the REALSXP
+	       avoids an allocation. */
+	    return s;
 	}
     }
 
-    PROTECT(s = duplicate(s));
-    SET_ATTRIB(s, R_NilValue);
-    SET_OBJECT(s, 0);
-
+    R_xlen_t ns = xlength(s);
     SEXP ans = R_NilValue;
     switch (TYPEOF(s)) {
     case NILSXP:
@@ -900,16 +1085,17 @@ makeSubscript(SEXP x, SEXP s, R_xlen_t *stretch, SEXP call)
 	ans = logicalSubscript(s, ns, nx, stretch, call);
 	break;
     case INTSXP:
-	ans = integerSubscript(s, ns, nx, stretch, call);
+	ans = integerSubscript(s, ns, nx, stretch, call, x);
 	break;
     case REALSXP:
-	ans = realSubscript(s, ns, nx, stretch, call);
+	ans = realSubscript(s, ns, nx, stretch, call, x);
 	break;
     case STRSXP:
     {
-	SEXP names = getAttrib(x, R_NamesSymbol);
+	SEXP names = PROTECT(getAttrib(x, R_NamesSymbol));
 	/* *stretch = 0; */
-	ans = stringSubscript(s, ns, nx, names, stretch, call);
+	ans = stringSubscript(s, ns, nx, names, stretch, call, x, -1);
+	UNPROTECT(1); /* names */
 	break;
     }
     case SYMSXP:
@@ -919,12 +1105,7 @@ makeSubscript(SEXP x, SEXP s, R_xlen_t *stretch, SEXP call)
 	    break;
 	}
     default:
-	if (call == R_NilValue)
-	    error(_("invalid subscript type '%s'"), type2char(TYPEOF(s)));
-	else
-	    errorcall(call, _("invalid subscript type '%s'"),
-		      type2char(TYPEOF(s)));
+	ECALL3(call, _("invalid subscript type '%s'"), R_typeToChar(s));
     }
-    UNPROTECT(1);
     return ans;
 }
